#!/usr/bin/env python

import threading
import time

import numpy

from gnuradio import gr, gr_unittest, blocks


class add_ff(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="add_ff",
            in_sig=[numpy.float32, numpy.float32],
            out_sig=[numpy.float32],
        )

    def work(self, input_items, output_items):
        output_items[0][:] = input_items[0] + input_items[1]
        return len(output_items[0])


class multiply_const_ff(gr.sync_block):
    def __init__(self, k):
        gr.sync_block.__init__(
            self,
            name="multiply_ff",
            in_sig=[numpy.float32],
            out_sig=[numpy.float32],
        )
        self.k = k

    def work(self, input_items, output_items):
        output_items[0][:] = [self.k * x for x in input_items[0]]
        return len(output_items[0])


class test_blocks_hier_block2(gr_unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_001_make(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        self.assertEqual("test_block", hblock.name())
        self.assertEqual(1, hblock.input_signature().max_streams())
        self.assertEqual(1, hblock.output_signature().min_streams())
        self.assertEqual(1, hblock.output_signature().max_streams())
        self.assertEqual(
            gr.sizeof_int,
            hblock.output_signature().sizeof_stream_item(0))

    def test_002_connect_input(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(hblock, nop1)

    def test_004_connect_output(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(nop1, hblock)

    def test_005_connect_output_in_use(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        nop2 = blocks.nop(gr.sizeof_int)
        hblock.connect(nop1, hblock)
        self.assertRaises(ValueError,
                          lambda: hblock.connect(nop2, hblock))

    def test_006_connect_invalid_src_port_neg(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        self.assertRaises(ValueError,
                          lambda: hblock.connect((hblock, -1), nop1))

    def test_005_connect_invalid_src_port_exceeds(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        self.assertRaises(ValueError,
                          lambda: hblock.connect((hblock, 1), nop1))

    def test_007_connect_invalid_dst_port_neg(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        nop2 = blocks.nop(gr.sizeof_int)
        self.assertRaises(ValueError,
                          lambda: hblock.connect(nop1, (nop2, -1)))

    def test_008_connect_invalid_dst_port_exceeds(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.null_sink(gr.sizeof_int)
        nop2 = blocks.null_sink(gr.sizeof_int)
        self.assertRaises(ValueError,
                          lambda: hblock.connect(nop1, (nop2, 1)))

    def test_009_check_topology(self):
        hblock = gr.top_block("test_block")
        hblock.check_topology(0, 0)

    def test_010_run(self):
        expected = [1.0, 2.0, 3.0, 4.0]
        hblock = gr.top_block("test_block")
        src = blocks.vector_source_f(expected, False)
        sink1 = blocks.vector_sink_f()
        sink2 = blocks.vector_sink_f()
        hblock.connect(src, sink1)
        hblock.connect(src, sink2)
        hblock.run()
        actual1 = sink1.data()
        actual2 = sink2.data()
        self.assertEqual(expected, actual1)
        self.assertEqual(expected, actual2)

    def test_012_disconnect_input(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(hblock, nop1)
        hblock.disconnect(hblock, nop1)

    def test_013_disconnect_input_not_connected(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        nop2 = blocks.nop(gr.sizeof_int)
        hblock.connect(hblock, nop1)
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect(hblock, nop2))

    def test_014_disconnect_input_neg(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(hblock, nop1)
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect((hblock, -1), nop1))

    def test_015_disconnect_input_exceeds(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(hblock, nop1)
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect((hblock, 1), nop1))

    def test_016_disconnect_output(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(nop1, hblock)
        hblock.disconnect(nop1, hblock)

    def test_017_disconnect_output_not_connected(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        nop2 = blocks.nop(gr.sizeof_int)
        hblock.connect(nop1, hblock)
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect(nop2, hblock))

    def test_018_disconnect_output_neg(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(hblock, nop1)
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect(nop1, (hblock, -1)))

    def test_019_disconnect_output_exceeds(self):
        hblock = gr.hier_block2("test_block",
                                gr.io_signature(1, 1, gr.sizeof_int),
                                gr.io_signature(1, 1, gr.sizeof_int))
        nop1 = blocks.nop(gr.sizeof_int)
        hblock.connect(nop1, hblock)
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect(nop1, (hblock, 1)))

    def test_020_run(self):
        hblock = gr.top_block("test_block")
        data = [1.0, 2.0, 3.0, 4.0]
        src = blocks.vector_source_f(data, False)
        dst = blocks.vector_sink_f()
        hblock.connect(src, dst)
        hblock.run()
        self.assertEqual(data, dst.data())

    def test_021_connect_single(self):
        hblock = gr.top_block("test_block")
        blk = gr.hier_block2("block",
                             gr.io_signature(0, 0, 0),
                             gr.io_signature(0, 0, 0))
        hblock.connect(blk)

    def test_022_connect_single_with_ports(self):
        hblock = gr.top_block("test_block")
        blk = gr.hier_block2("block",
                             gr.io_signature(1, 1, 1),
                             gr.io_signature(1, 1, 1))
        self.assertRaises(ValueError,
                          lambda: hblock.connect(blk))

    def test_023_connect_single_twice(self):
        hblock = gr.top_block("test_block")
        blk = gr.hier_block2("block",
                             gr.io_signature(0, 0, 0),
                             gr.io_signature(0, 0, 0))
        hblock.connect(blk)
        self.assertRaises(ValueError,
                          lambda: hblock.connect(blk))

    def test_024_disconnect_single(self):
        hblock = gr.top_block("test_block")
        blk = gr.hier_block2("block",
                             gr.io_signature(0, 0, 0),
                             gr.io_signature(0, 0, 0))
        hblock.connect(blk)
        hblock.disconnect(blk)

    def test_025_disconnect_single_not_connected(self):
        hblock = gr.top_block("test_block")
        blk = gr.hier_block2("block",
                             gr.io_signature(0, 0, 0),
                             gr.io_signature(0, 0, 0))
        self.assertRaises(ValueError,
                          lambda: hblock.disconnect(blk))

    def test_026_run_single(self):
        expected_data = [1.0, ]
        tb = gr.top_block("top_block")
        hb = gr.hier_block2("block",
                            gr.io_signature(0, 0, 0),
                            gr.io_signature(0, 0, 0))
        src = blocks.vector_source_f(expected_data)
        dst = blocks.vector_sink_f()
        hb.connect(src, dst)
        tb.connect(hb)
        tb.run()
        self.assertEqual(expected_data, dst.data())

    def test_027a_internally_unconnected_input(self):
        tb = gr.top_block()
        hb = gr.hier_block2("block",
                            gr.io_signature(1, 1, 1),
                            gr.io_signature(1, 1, 1))
        hsrc = blocks.vector_source_b([1, ])
        hb.connect(hsrc, hb)  # wire output internally
        src = blocks.vector_source_b([1, ])
        dst = blocks.vector_sink_b()
        tb.connect(src, hb, dst)  # hb's input is not connected internally
        self.assertRaises(RuntimeError,
                          lambda: tb.run())

    def test_027b_internally_unconnected_output(self):
        tb = gr.top_block()

        hb = gr.hier_block2("block",
                            gr.io_signature(1, 1, 1),
                            gr.io_signature(1, 1, 1))
        hdst = blocks.vector_sink_b()
        hb.connect(hb, hdst)  # wire input internally
        src = blocks.vector_source_b([1, ])
        dst = blocks.vector_sink_b()
        tb.connect(src, hb, dst)  # hb's output is not connected internally
        self.assertRaises(RuntimeError,
                          lambda: tb.run())

    def test_027c_fully_unconnected_output(self):
        tb = gr.top_block()
        hb = gr.hier_block2("block",
                            gr.io_signature(1, 1, 1),
                            gr.io_signature(1, 1, 1))
        hsrc = blocks.vector_sink_b()
        hb.connect(hb, hsrc)  # wire input internally
        src = blocks.vector_source_b([1, ])
        dst = blocks.vector_sink_b()
        # hb's output is not connected internally or externally
        tb.connect(src, hb)
        self.assertRaises(RuntimeError,
                          lambda: tb.run())

    def test_027d_fully_unconnected_input(self):
        tb = gr.top_block()
        hb = gr.hier_block2("block",
                            gr.io_signature(1, 1, 1),
                            gr.io_signature(1, 1, 1))
        hdst = blocks.vector_source_b([1, ])
        hb.connect(hdst, hb)  # wire output internally
        dst = blocks.vector_sink_b()
        # hb's input is not connected internally or externally
        tb.connect(hb, dst)
        self.assertRaises(RuntimeError,
                          lambda: tb.run())

    def test_028_singleton_reconfigure(self):
        tb = gr.top_block()
        hb = gr.hier_block2("block",
                            gr.io_signature(0, 0, 0), gr.io_signature(0, 0, 0))
        src = blocks.vector_source_b([1, ])
        dst = blocks.vector_sink_b()
        hb.connect(src, dst)
        tb.connect(hb)  # Singleton connect
        tb.lock()
        tb.disconnect_all()
        tb.connect(src, dst)
        tb.unlock()

    def test_029_singleton_disconnect(self):
        tb = gr.top_block()
        src = blocks.vector_source_b([1, ])
        dst = blocks.vector_sink_b()
        tb.connect(src, dst)
        tb.disconnect(src)   # Singleton disconnect
        tb.connect(src, dst)
        tb.run()
        self.assertEqual(dst.data(), [1, ])

    def test_030_nested_input(self):
        tb = gr.top_block()
        src = blocks.vector_source_b([1, ])
        hb1 = gr.hier_block2("hb1",
                             gr.io_signature(1, 1, gr.sizeof_char),
                             gr.io_signature(0, 0, 0))
        hb2 = gr.hier_block2("hb2",
                             gr.io_signature(1, 1, gr.sizeof_char),
                             gr.io_signature(0, 0, 0))
        dst = blocks.vector_sink_b()
        tb.connect(src, hb1)
        hb1.connect(hb1, hb2)
        hb2.connect(hb2, blocks.copy(gr.sizeof_char), dst)
        tb.run()
        self.assertEqual(dst.data(), [1, ])

    def test_031_multiple_internal_inputs(self):
        tb = gr.top_block()
        src = blocks.vector_source_f([1.0, ])
        hb = gr.hier_block2("hb",
                            gr.io_signature(1, 1, gr.sizeof_float),
                            gr.io_signature(1, 1, gr.sizeof_float))
        m1 = multiply_const_ff(1.0)
        m2 = multiply_const_ff(2.0)
        add = add_ff()
        hb.connect(hb, m1)       # m1 is connected to hb external input #0
        hb.connect(hb, m2)       # m2 is also connected to hb external input #0
        hb.connect(m1, (add, 0))
        hb.connect(m2, (add, 1))
        hb.connect(add, hb)      # add is connected to hb external output #0
        dst = blocks.vector_sink_f()
        tb.connect(src, hb, dst)
        tb.run()
        self.assertEqual(dst.data(), [3.0, ])

    def test_032_nested_multiple_internal_inputs(self):
        tb = gr.top_block()
        src = blocks.vector_source_f([1.0, ])
        hb = gr.hier_block2("hb",
                            gr.io_signature(1, 1, gr.sizeof_float),
                            gr.io_signature(1, 1, gr.sizeof_float))
        hb2 = gr.hier_block2("hb",
                             gr.io_signature(1, 1, gr.sizeof_float),
                             gr.io_signature(1, 1, gr.sizeof_float))

        m1 = multiply_const_ff(1.0)
        m2 = multiply_const_ff(2.0)
        add = add_ff()
        hb2.connect(hb2, m1)       # m1 is connected to hb2 external input #0
        # m2 is also connected to hb2 external input #0
        hb2.connect(hb2, m2)
        hb2.connect(m1, (add, 0))
        hb2.connect(m2, (add, 1))
        hb2.connect(add, hb2)      # add is connected to hb2 external output #0
        hb.connect(hb, hb2, hb)   # hb as hb2 as nested internal block
        dst = blocks.vector_sink_f()
        tb.connect(src, hb, dst)
        tb.run()
        self.assertEqual(dst.data(), [3.0, ])

    def test_033a_set_affinity(self):
        expected = [1.0, 2.0, 3.0, 4.0]
        hblock = gr.top_block("test_block")
        src = blocks.vector_source_f(expected, False)
        snk = blocks.vector_sink_f()
        hblock.connect(src, snk)
        hblock.set_processor_affinity([0, ])
        hblock.run()
        actual = snk.data()
        self.assertEqual(expected, actual)

    def test_033b_unset_affinity(self):
        expected = [1.0, 2.0, 3.0, 4.0]
        hblock = gr.top_block("test_block")
        src = blocks.vector_source_f(expected, False)
        snk = blocks.vector_sink_f()
        hblock.connect(src, snk)
        hblock.set_processor_affinity([0, ])
        hblock.unset_processor_affinity()
        hblock.run()
        actual = snk.data()
        self.assertEqual(expected, actual)

    def test_033c_get_affinity(self):
        expected = [1.0, 2.0, 3.0, 4.0]
        hblock = gr.top_block("test_block")
        src = blocks.vector_source_f(expected, False)
        snk = blocks.vector_sink_f()
        hblock.connect(src, snk)
        hblock.set_processor_affinity([0, ])
        procs = hblock.processor_affinity()
        self.assertEqual([0, ], procs)

    def test_34a_lock_unlock(self):
        hblock = gr.top_block("test_block")
        src = blocks.null_source(gr.sizeof_float)
        throttle = blocks.throttle(gr.sizeof_float, 32000)
        hier = multiply_const_ff(0.5)
        sink = blocks.null_sink(gr.sizeof_float)
        hblock.connect(src, throttle, hier, sink)
        hblock.set_processor_affinity([0, ])
        hblock.start()
        time.sleep(1)
        hblock.lock()
        hblock.unlock()
        hblock.stop()
        hblock.wait()

    def test_34b_lock_unlock(self):
        hblock = gr.top_block("test_block")
        src = blocks.null_source(gr.sizeof_float)
        throttle = blocks.throttle(gr.sizeof_float, 32000)
        sink = blocks.null_sink(gr.sizeof_float)
        hblock.connect(src, throttle, sink)
        hblock.set_processor_affinity([0, ])

        def thread_01(hblock, cls):
            cls.test_34b_val = 10
            hblock.lock()
            cls.test_34b_val = 20
            hblock.unlock()
            cls.test_34b_val = 30
            time.sleep(0.5)
            cls.test_34b_val = 40
            hblock.stop()
        hblock.start()
        self.test_34b_val = 0
        t1 = threading.Thread(target=thread_01, args=(hblock, self, ))
        t1.start()
        hblock.wait()
        self.assertEqual(40, self.test_34b_val)

    def test_035_lock_after_disconnect(self):
        tb = gr.top_block()
        src = blocks.vector_source_f([1.0, ])
        hb = gr.hier_block2("hb",
                            gr.io_signature(1, 1, gr.sizeof_float),
                            gr.io_signature(1, 1, gr.sizeof_float))
        m1 = multiply_const_ff(1.0)
        hb.connect(hb, m1)
        hb.connect(m1, hb)
        dst1 = blocks.vector_sink_f()
        dst2 = blocks.vector_sink_f()
        tb.connect(src, hb, dst1)
        tb.connect(hb, dst2)
        tb.run()
        tb.disconnect(hb, dst2)
        hb.lock()
        hb.unlock()
        tb.run()

    def test_036_unconnected_lock(self):
        hb = gr.hier_block2("hb",
                            gr.io_signature(1, 1, gr.sizeof_float),
                            gr.io_signature(1, 1, gr.sizeof_float))
        hb.lock()
        hb.unlock()

    def test_037_crossparent(self):
        hb = gr.hier_block2("hb",
                            gr.io_signature(1, 1, gr.sizeof_float),
                            gr.io_signature(1, 1, gr.sizeof_float))
        parent0 = gr.hier_block2("parent0",
                                 gr.io_signature(1, 1, gr.sizeof_float),
                                 gr.io_signature(1, 1, gr.sizeof_float))
        parent1 = gr.hier_block2("parent1",
                                 gr.io_signature(1, 1, gr.sizeof_float),
                                 gr.io_signature(1, 1, gr.sizeof_float))
        parent0.connect(parent0, hb)
        self.assertRaises(RuntimeError,
                          lambda: parent1.connect(hb, parent1))


if __name__ == "__main__":
    gr_unittest.run(test_blocks_hier_block2)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks


class test_probe_density(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [0, 1, 0, 1]
        expected_data = 1
        src = blocks.vector_source_b(src_data)
        op = digital.probe_density_b(1)
        self.tb.connect(src, op)
        self.tb.run()

        result_data = op.density()
        self.assertEqual(expected_data, result_data)

    def test_002(self):
        src_data = [1, 1, 1, 1]
        expected_data = 1
        src = blocks.vector_source_b(src_data)
        op = digital.probe_density_b(0.01)
        self.tb.connect(src, op)
        self.tb.run()

        result_data = op.density()
        self.assertEqual(expected_data, result_data)

    def test_003(self):
        src_data = [0, 1, 0, 1, 0, 1, 0, 1, 0, 1]
        expected_data = 0.95243
        src = blocks.vector_source_b(src_data)
        op = digital.probe_density_b(0.01)
        self.tb.connect(src, op)
        self.tb.run()

        result_data = op.density()
        print(result_data)
        self.assertAlmostEqual(expected_data, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_probe_density)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010-2013,2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks
import numpy as np


class test_constellation_encoder(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_constellation_encoder_bc_bpsk(self):
        cnst = digital.constellation_bpsk()

        src_data = (1, 1, 0, 0,
                    1, 0, 1)
        const_map = [-1.0, 1.0]
        expected_result = [const_map[x] for x in src_data]

        src = blocks.vector_source_b(src_data)
        op = digital.constellation_encoder_bc(cnst.base())
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)

    def test_constellation_encoder_bc_qpsk(self):
        cnst = digital.constellation_qpsk()
        src_data = (3, 1, 0, 2,
                    3, 2, 1)
        expected_result = [cnst.points()[x] for x in src_data]
        src = blocks.vector_source_b(src_data)
        op = digital.constellation_encoder_bc(cnst.base())
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)

    def test_constellation_encoder_bc_qpsk_random(self):
        cnst = digital.constellation_qpsk()
        src_data = np.random.randint(0, 4, size=20000)
        expected_result = [cnst.points()[x] for x in src_data]
        src = blocks.vector_source_b(src_data)
        op = digital.constellation_encoder_bc(cnst.base())
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_constellation_encoder)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import unittest
import pmt
from pmt import pmt_to_python as pmt2py


class test_pmt_to_python(unittest.TestCase):

    def test_pmt_from_double(self):
        b = pmt.from_double(123765)
        self.assertEqual(pmt.to_python(b), 123765)
        t = pmt.to_pmt(list(range(5)))

    def test_numpy_to_uvector_and_reverse(self):
        import numpy as np
        N = 100
        narr = np.ndarray(N, dtype=np.cdouble)
        narr.real[:] = np.random.uniform(size=N)
        narr.imag[:] = np.random.uniform(size=N)
        uvector = pmt2py.numpy_to_uvector(narr)
        nparr = pmt2py.uvector_to_numpy(uvector)
        self.assertEqual(nparr.dtype, narr.dtype)
        self.assertTrue(np.all(nparr == narr))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2021 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import numpy as np
from numpy.random import uniform
import pmt


class qa_rotator_cc(gr_unittest.TestCase):

    def _setUp(self, n_samples=100, tag_inc_updates=True):
        """Base fixture: set up flowgraph and parameters"""
        self.n_samples = n_samples  # number of IQ samples to generate
        self.f_in = uniform(high=0.5)  # input frequency
        self.f_shift = uniform(high=0.5) - \
            self.f_in  # rotator's starting frequency

        # Input IQ samples
        in_angles = 2 * np.pi * np.arange(self.n_samples) * self.f_in
        in_samples = np.exp(1j * in_angles)

        # Rotator's starting phase increment
        phase_inc = 2 * np.pi * self.f_shift

        # Flowgraph:
        #
        #                             /-> Vector Sink
        # Vector Source -> Rotator --/
        #                            \
        #                             \-> Tag Debug
        #
        self.tb = gr.top_block()
        self.source = blocks.vector_source_c(in_samples)
        self.throttle = blocks.throttle(gr.sizeof_gr_complex, 2**16)
        self.rotator_cc = blocks.rotator_cc(phase_inc, tag_inc_updates)
        self.sink = blocks.vector_sink_c()
        self.tag_sink = blocks.tag_debug(gr.sizeof_gr_complex, "rot_phase_inc",
                                         "rot_phase_inc")
        self.tag_sink.set_save_all(True)
        self.tb.connect(self.source, self.throttle, self.rotator_cc, self.sink)
        self.tb.connect(self.rotator_cc, self.tag_sink)

    def setUp(self):
        self._setUp()

    def tearDown(self):
        self.tb = None

    def _post_phase_inc_cmd(self, new_phase_inc, offset=None):
        """Post phase increment update command to the rotator block"""
        cmd = pmt.make_dict()
        cmd = pmt.dict_add(cmd, pmt.intern("inc"),
                           pmt.from_double(new_phase_inc))
        if (offset is not None):
            cmd = pmt.dict_add(cmd, pmt.intern("offset"),
                               pmt.from_uint64(offset))
        self.rotator_cc.insert_tail(pmt.to_pmt("cmd"), cmd)

    def _assert_tags(self, expected_values, expected_offsets):
        """Check the tags received by the tag debug block"""
        tags = self.tag_sink.current_tags()
        expected_tags = list(zip(expected_values, expected_offsets))
        self.assertEqual(len(tags), len(expected_tags))

        for idx, (val, offset) in enumerate(expected_tags):
            self.assertAlmostEqual(pmt.to_double(tags[idx].value),
                                   val,
                                   places=5)
            self.assertEqual(tags[idx].offset, offset)

    def _compute_expected_samples(self, offsets, new_phase_incs):
        """Compute the samples expected on the rotator output

        Args:
            offsets (list): Sample offsets where the updates are expected.
            new_phase_incs (list): Rotator phase increments for each update.

        Returns:
            np.array: Array of expected IQ samples on the rotator output.

        """
        # Initial samples (preceding the first phase increment update)
        f_out = self.f_in + self.f_shift
        expected_angles = 2 * np.pi * np.arange(offsets[0]) * f_out
        expected_samples = np.exp(1j * expected_angles)

        for idx, (offset,
                  rot_phase_inc) in enumerate(zip(offsets, new_phase_incs)):
            prev_f_out = f_out  # Previous output frequency
            new_f_shift = rot_phase_inc / (2.0 * np.pi)  # New frequency shift
            f_out = self.f_in + new_f_shift  # New output frequency

            if idx == len(offsets) - 1:
                segment_len = self.n_samples - offset
            else:
                segment_len = offsets[idx + 1] - offset

            # NOTE: when the phase increment is updated, the assumption is that
            # the update occurs at the beginning of the sample period. Hence,
            # the new phase rotation pace is only observed in the succeeding
            # sample, at the beginning of the next sample period. In other
            # words, the new phase increment can only be observed after some
            # time goes by such that the rotator phase accumulates.
            #
            # For example, suppose there are ten samples, and the update occurs
            # at offset 5 (i.e., the sixth sample). Suppose also that the
            # initial phase increment is 0.1, and the new increment is 0.2. In
            # this case, we have the following sequence of angles:
            #
            #    [0, 0.1, 0.2, 0.3, 0.4, 0.5. 0.7, 0.9, 1.1, 1.3]
            #                             |     \
            #                             |      \--------\
            #                           update             \
            #                           applied         new phase
            #                                        increment observed
            #
            # Ultimately, the phase seen at the sample where the update is
            # applied is still determined by the preceding phase increment.
            expected_angles = expected_angles[-1] + (2 * np.pi * prev_f_out) +\
                (2 * np.pi * np.arange(segment_len) * f_out)
            expected_samples = np.concatenate(
                (expected_samples, np.exp(1j * expected_angles)))

        return expected_samples

    def _post_random_phase_inc_updates(self, offsets):
        """Update the phase increment randomly at chosen offsets

        Args:
            offsets (list): Sample offsets where the updates are to be applied.

        Returns:
            list: New phase increments defined randomly (list).

        """
        new_phase_incs = list()
        for offset in offsets:
            new_f_out = uniform(high=0.5)
            new_f_shift = new_f_out - self.f_in
            new_phase_inc = float(2 * np.pi * new_f_shift)
            new_phase_incs.append(new_phase_inc)
            self._post_phase_inc_cmd(new_phase_inc, offset)

        return new_phase_incs

    def test_freq_shift(self):
        """Complex sinusoid frequency shift"""
        f_out = self.f_in + self.f_shift  # expected output frequency

        # Expected IQ samples
        expected_angles = 2 * np.pi * np.arange(self.n_samples) * f_out
        expected_samples = np.exp(1j * expected_angles)

        self.tb.run()
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_scheduled_phase_inc_update(self):
        """Update the phase increment at a chosen offset via command message"""
        offset = int(self.n_samples / 2)  # when to update the phase increment
        offsets = [offset]
        new_phase_incs = self._post_random_phase_inc_updates(offsets)
        expected_samples = self._compute_expected_samples(
            offsets, new_phase_incs)

        self.tb.run()
        self._assert_tags(new_phase_incs, offsets)
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_scheduled_phase_inc_update_with_tagging_disabled(self):
        """Test a scheduled phase increment update without tagging the update

        Same as test_scheduled_phase_inc_update but with tagging disabled.

        """
        self._setUp(tag_inc_updates=False)

        offset = int(self.n_samples / 2)  # when to update the phase increment
        offsets = [offset]
        new_phase_incs = self._post_random_phase_inc_updates(offsets)
        expected_samples = self._compute_expected_samples(
            offsets, new_phase_incs)

        self.tb.run()
        tags = self.tag_sink.current_tags()
        self.assertEqual(len(tags), 0)
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_immediate_phase_inc_update(self):
        """Immediate phase increment update via command message

        In this test, the command message does not include the offset
        key. Hence, the rotator should update its phase increment immediately.

        """
        new_f_shift = uniform(high=0.5) - self.f_in  # rotator's new frequency
        new_phase_inc = float(2 * np.pi * new_f_shift)  # new phase increment
        f_out = self.f_in + new_f_shift  # output frequency after update

        # Post the phase increment command message to the rotator block
        self._post_phase_inc_cmd(new_phase_inc)

        # The rotator updates the increment immediately (on the first sample)
        expected_tag_offset = 0

        # Expected samples (all of them with the new frequency set via message)
        expected_angles = 2 * np.pi * np.arange(self.n_samples) * f_out
        expected_samples = np.exp(1j * expected_angles)

        self.tb.run()
        self._assert_tags([new_phase_inc], [expected_tag_offset])
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_zero_change_phase_inc_update(self):
        """Schedule a phase increment update that does not change anything

        If the scheduled phase increment update sets the same phase increment
        that is already active in the rotator block, there should be no effect
        on the output signal. Nevertheless, the rotator should still tag the
        update.

        """
        new_phase_inc = 2 * np.pi * self.f_shift  # no change
        offset = int(self.n_samples / 2)  # when to update the phase increment
        f_out = self.f_in + self.f_shift  # expected output frequency

        # Post the phase increment command message to the rotator block
        self._post_phase_inc_cmd(new_phase_inc, offset)

        # Expected IQ samples
        expected_angles = 2 * np.pi * np.arange(self.n_samples) * f_out
        expected_samples = np.exp(1j * expected_angles)

        self.tb.run()
        self._assert_tags([new_phase_inc], [offset])
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_consecutive_phase_inc_updates(self):
        """Test tagging of a few consecutive phase increment updates"""
        offsets = list(map(int, self.n_samples * np.arange(1, 4, 1) / 4))
        new_phase_incs = self._post_random_phase_inc_updates(offsets)
        expected_samples = self._compute_expected_samples(
            offsets, new_phase_incs)

        self.tb.run()
        self._assert_tags(new_phase_incs, offsets)
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_out_of_order_phase_inc_updates(self):
        """Test tagging of a few out-of-order phase increment updates

        The rotator should sort the increment updates and apply them in order.

        """
        n_updates = 3
        new_f_shifts = uniform(high=0.5, size=n_updates)  # new frequencies
        new_phase_incs = 2 * np.pi * new_f_shifts  # new phase increments
        offsets = self.n_samples * np.arange(1, 4, 1) / 4  # when to update

        # Post the phase increment command messages out of order
        for i in [0, 2, 1]:
            self._post_phase_inc_cmd(new_phase_incs[i], int(offsets[i]))

        self.tb.run()

        # Confirm they are received in order
        self._assert_tags(new_phase_incs, offsets)

        # Confirm the output waveform
        expected_samples = self._compute_expected_samples(
            offsets, new_phase_incs)
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_duplicate_phase_inc_updates(self):
        """Test multiple phase increment updates scheduled for the same sample

        The rotator block applies all updates scheduled for the same sample
        offset. In the end, only the last update shall take effect.

        """
        n_updates = 3
        offset = int(self.n_samples / 2)
        all_new_phase_incs = list()
        # Post the phase increment command messages
        for i in range(n_updates):
            new_phase_incs = self._post_random_phase_inc_updates([offset])
            expected_samples = self._compute_expected_samples([offset],
                                                              new_phase_incs)
            all_new_phase_incs.extend(new_phase_incs)

        self.tb.run()

        # All "n_updates" tags are expected to be present
        self._assert_tags(all_new_phase_incs, [offset] * n_updates)

        # However, only the last update takes effect on the rotated samples
        self.assertComplexTuplesAlmostEqual(self.sink.data(),
                                            expected_samples,
                                            places=4)

    def test_phase_inc_update_out_of_range(self):
        """Test phase increment update sent for an out-of-range offset"""
        self._setUp(n_samples=2**16)
        n_half_samples = int(self.n_samples / 2)

        # Schedule a phase increment update whose offset is initially
        # out-of-range due to being in the future
        new_phase_inc = 2 * np.pi * 0.1
        self._post_phase_inc_cmd(new_phase_inc, offset=n_half_samples)

        # Run the flowgraph and wait until the rotator block does some work
        self.tb.start()
        while (self.rotator_cc.nitems_written(0) == 0):
            pass

        # The out-of-range update (scheduled for the future) should not have
        # been applied yet, but it should not have been discarded either.
        self._assert_tags([], [])

        # Wait until at least the first half of samples have been processed
        while (self.rotator_cc.nitems_written(0) < n_half_samples):
            pass

        # Now, schedule an update that is out-of-range due to being in the past
        self._post_phase_inc_cmd(new_phase_inc, offset=0)

        # And run the flowgraph to completion
        self.tb.wait()

        # The increment update initially scheduled for the future should have
        # been applied when processing the second half of samples. In contrast,
        # the update scheduled for the past should have been discarded.
        self._assert_tags([new_phase_inc], [n_half_samples])


if __name__ == '__main__':
    gr_unittest.run(qa_rotator_cc)



#
# Copyright 2013, 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
''' A parser for blocks written in C++ '''


import re
import sys
import logging

logger = logging.getLogger(__name__)


def dummy_translator(the_type, default_v=None):
    """ Doesn't really translate. """
    return the_type


def remove_comments(inp: str) -> str:
    def comment_removal(match: re.match) -> str:
        groups = match.groups()
        if len(groups) == 1:
            return ''
        return inp
    try:
        out = re.sub(r'(/\*[^(\*/)]*\*/)', comment_removal, inp)
        logger.debug(f"out: {out}")
        return out
    except Exception as e:
        logger.error(f"error removing comments {e}")
        return inp


class ParserCCBlock(object):
    """ Class to read blocks written in C++ """

    def __init__(self, filename_cc, filename_h, blockname, version, type_trans=dummy_translator):
        with open(filename_cc) as f:
            self.code_cc = f.read()
        with open(filename_h) as f:
            self.code_h = f.read()
        self.blockname = blockname
        self.type_trans = type_trans
        self.version = version

    def read_io_signature(self):
        """ Scans a .cc file for an IO signature. """
        def _figure_out_iotype_and_vlen(iosigcall, typestr):
            """ From a type identifier, returns the data type.
            E.g., for sizeof(int), it will return 'int'.
            Returns a list! """
            if 'gr::io_signature::makev' in iosigcall:
                logger.error('tbi')
                raise ValueError
            return {'type': [_typestr_to_iotype(x) for x in typestr.split(',')],
                    'vlen': [_typestr_to_vlen(x) for x in typestr.split(',')]
                    }

        def _typestr_to_iotype(typestr):
            """ Convert a type string (e.g. sizeof(int) * vlen) to the type (e.g. 'int'). """
            type_match = re.search(r'sizeof\s*\(([^)]*)\)', typestr)
            if type_match is None:
                return self.type_trans('char')
            return self.type_trans(remove_comments(type_match.group(1)))

        def _typestr_to_vlen(typestr):
            """ From a type identifier, returns the vector length of the block's
            input/out. E.g., for 'sizeof(int) * 10', it returns 10. For
            'sizeof(int)', it returns '1'. For 'sizeof(int) * vlen', it returns
            the string vlen. """
            # Catch fringe case where no sizeof() is given
            if typestr.find('sizeof') == -1:
                return typestr
            if typestr.find('*') == -1:
                return '1'
            vlen_parts = typestr.split('*')
            for fac in vlen_parts:
                if fac.find('sizeof') != -1:
                    vlen_parts.remove(fac)
            if len(vlen_parts) == 1:
                return vlen_parts[0].strip()
            elif len(vlen_parts) > 1:
                return '*'.join(vlen_parts).strip()
        iosig = {}
        iosig_regex = r'(?P<incall>(gr::)?io_signature::make[23v]?)\s*\(\s*(?P<inmin>[^,]+),\s*(?P<inmax>[^,]+),' + \
                      r'\s*(?P<intype>(\([^\)]*\)|[^)])+)\),\s*' + \
                      r'(?P<outcall>(gr::)?io_signature::make[23v]?)\s*\(\s*(?P<outmin>[^,]+),\s*(?P<outmax>[^,]+),' + \
                      r'\s*(?P<outtype>(\([^\)]*\)|[^)])+)\)'
        iosig_match = re.compile(
            iosig_regex, re.MULTILINE).search(self.code_cc)

        try:
            iosig['in'] = _figure_out_iotype_and_vlen(iosig_match.group('incall'),
                                                      iosig_match.group('intype'))
            iosig['in']['min_ports'] = remove_comments(iosig_match.group('inmin'))
            iosig['in']['max_ports'] = remove_comments(iosig_match.group('inmax'))
        except Exception as e:
            logger.error(f"Error: Can't parse input signature: {e}")
        try:
            iosig['out'] = _figure_out_iotype_and_vlen(iosig_match.group('outcall'),
                                                       iosig_match.group('outtype'))
            iosig['out']['min_ports'] = remove_comments(iosig_match.group('outmin'))
            iosig['out']['max_ports'] = remove_comments(iosig_match.group('outmax'))
        except Exception as e:
            logger.error(f"Error: Can't parse output signature: {e}")
        return iosig

    def read_params(self):
        """ Read the parameters required to initialize the block """
        def _scan_param_list(start_idx):
            """ Go through a parameter list and return a tuple each:
                (type, name, default_value). Python's re just doesn't cut
                it for C++ code :( """
            i = start_idx
            c = self.code_h
            if c[i] != '(':
                raise ValueError
            i += 1

            param_list = []
            read_state = 'type'
            in_string = False
            parens_count = 0  # Counts ()
            brackets_count = 0  # Counts <>
            end_of_list = False
            this_type = ''
            this_name = ''
            this_defv = ''
            WHITESPACE = ' \t\n\r\f\v'
            while not end_of_list:
                # Keep track of (), stop when reaching final closing parens
                if not in_string:
                    if c[i] == ')':
                        if parens_count == 0:
                            if read_state == 'type' and len(this_type):
                                raise ValueError(
                                    'Found closing parentheses before finishing '
                                    'last argument (this is how far I got: {})'.format
                                    (str(param_list))
                                )
                            if len(this_type):
                                param_list.append(
                                    (this_type, this_name, this_defv))
                            end_of_list = True
                            break
                        else:
                            parens_count -= 1
                    elif c[i] == '(':
                        parens_count += 1
                # Parameter type (int, const std::string, std::vector<gr_complex>, unsigned long ...)
                if read_state == 'type':
                    if c[i] == '<':
                        brackets_count += 1
                    if c[i] == '>':
                        brackets_count -= 1
                    if c[i] == '&':
                        i += 1
                        continue
                    if c[i] in WHITESPACE and brackets_count == 0:
                        while c[i] in WHITESPACE:
                            i += 1
                            continue
                        if this_type == 'const' or this_type == '':  # Ignore this
                            this_type = ''
                        elif this_type == 'unsigned':  # Continue
                            this_type += ' '
                            continue
                        else:
                            read_state = 'name'
                        continue
                    this_type += c[i]
                    i += 1
                    continue
                # Parameter name
                if read_state == 'name':
                    if c[i] == '&' or c[i] in WHITESPACE:
                        i += 1
                    elif c[i] == '=':
                        if parens_count != 0:
                            raise ValueError(
                                'While parsing argument {} ({}): name finished but no closing parentheses.'.format
                                (len(param_list) + 1, this_type + ' ' + this_name)
                            )
                        read_state = 'defv'
                        i += 1
                    elif c[i] == ',':
                        if parens_count:
                            raise ValueError(
                                'While parsing argument {} ({}): name finished but no closing parentheses.'.format
                                (len(param_list) + 1, this_type + ' ' + this_name)
                            )
                        read_state = 'defv'
                    else:
                        this_name += c[i]
                        i += 1
                    continue
                # Default value
                if read_state == 'defv':
                    if in_string:
                        if c[i] == '"' and c[i - 1] != '\\':
                            in_string = False
                        else:
                            this_defv += c[i]
                    elif c[i] == ',':
                        if parens_count:
                            raise ValueError(
                                'While parsing argument {} ({}): default value finished but no closing parentheses.'.format
                                (len(param_list) + 1, this_type + ' ' + this_name)
                            )
                        read_state = 'type'
                        param_list.append((this_type, this_name, this_defv))
                        this_type = ''
                        this_name = ''
                        this_defv = ''
                    else:
                        this_defv += c[i]
                    i += 1
                    continue
            return param_list
        # Go, go, go!
        if self.version in ('37', '38', '39', '310'):
            make_regex = r'static\s+sptr\s+make\s*'
        else:
            make_regex = r'(?<=_API)\s+\w+_sptr\s+\w+_make_\w+\s*'
        make_match = re.compile(make_regex, re.MULTILINE).search(self.code_h)
        try:
            params_list = _scan_param_list(make_match.end(0))
        except ValueError as ve:
            logger.error("Can't parse the argument list: ", ve.args[0])
            sys.exit(0)
        params = []
        for plist in params_list:
            params.append({'type': self.type_trans(plist[0], plist[2]),
                           'key': plist[1],
                           'default': plist[2],
                           'in_constructor': True})
        return params



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
from gnuradio import network
import random
import pmt
import time


class qa_socket_pdu (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_creation_destruction(self):
        # Test that blocks can be created and destroyed without hanging
        port = str(random.Random().randint(0, 30000) + 10000)
        self.pdu_send = network.socket_pdu("UDP_CLIENT", "localhost", port)
        self.pdu_recv = network.socket_pdu("UDP_SERVER", "localhost", port)
        self.pdu_send = None
        self.pdu_recv = None

    def test_002_send_pdu_through_udp(self):
        # Send a PDU through a pair of UDP sockets
        port = str(random.Random().randint(0, 30000) + 10000)
        srcdata = (0x64, 0x6f, 0x67, 0x65)
        data = pmt.init_u8vector(srcdata.__len__(), srcdata)
        pdu_msg = pmt.cons(pmt.PMT_NIL, data)

        self.pdu_source = blocks.message_strobe(pdu_msg, 500)
        self.pdu_recv = network.socket_pdu("UDP_SERVER", "localhost", port)
        self.pdu_send = network.socket_pdu("UDP_CLIENT", "localhost", port)

        self.dbg = blocks.message_debug()

        self.tb.msg_connect(self.pdu_source, "strobe", self.pdu_send, "pdus")
        self.tb.msg_connect(self.pdu_recv, "pdus", self.dbg, "store")

        self.tb.start()
        time.sleep(1)
        self.tb.stop()
        self.tb.wait()
        self.pdu_send = None
        self.pdu_recv = None

        received = self.dbg.get_message(0)
        received_data = pmt.cdr(received)
        msg_data = []
        for i in range(4):
            msg_data.append(pmt.u8vector_ref(received_data, i))
        self.assertEqual(srcdata, tuple(msg_data))

    def test_004_tcp_sub_mtu(self):
        # Test that the TCP server can stream PDUs <= the MTU size.
        port = str(random.Random().randint(0, 30000) + 10000)
        mtu = 10000
        srcdata = tuple(x % 256 for x in range(mtu))
        data = pmt.init_u8vector(srcdata.__len__(), srcdata)
        pdu_msg = pmt.cons(pmt.PMT_NIL, data)

        self.pdu_source = blocks.message_strobe(pdu_msg, 500)
        self.pdu_send = network.socket_pdu(
            "TCP_SERVER", "localhost", port, mtu)
        self.pdu_recv = network.socket_pdu(
            "TCP_CLIENT", "localhost", port, mtu)
        self.pdu_sink = blocks.message_debug()

        self.tb.msg_connect(self.pdu_source, "strobe", self.pdu_send, "pdus")
        self.tb.msg_connect(self.pdu_recv, "pdus", self.pdu_sink, "store")

        self.tb.start()
        time.sleep(1)
        self.tb.stop()
        self.tb.wait()

        received = self.pdu_sink.get_message(0)
        received_data = pmt.cdr(received)
        msg_data = []
        for i in range(mtu):
            msg_data.append(pmt.u8vector_ref(received_data, i))
        self.assertEqual(srcdata, tuple(msg_data))


if __name__ == '__main__':
    gr_unittest.run(qa_socket_pdu)



#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" testing the JSON files generated by gr-blocktool """


import sys
import json
import jsonschema

from blocktool import RESULT_SCHEMA


def is_valid():
    """ Validate json file """

    with open(sys.argv[1], 'r') as json_file:
        data = json.load(json_file)
    try:
        print("Validating...")
        jsonschema.validate(data, RESULT_SCHEMA)
    except jsonschema.ValidationError as exception:
        print("Record JSON file # {}: NOT OK".format(sys.argv[1]))
        raise Exception(exception)
    else:
        print("Record JSON file # {}: OK".format(sys.argv[1]))


if __name__ == '__main__':
    if len(sys.argv) == 2:
        is_valid()
    else:
        raise Exception('Please input only one json file')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2017 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import pmt


class qa_tag_share(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        # Constants
        tag_key = 'in1_tag'
        tag_value = 0
        tag_offset = 0
        in0_value = 1.0 + 1.0j
        in1_value = 2.717
        in0_data = [in0_value, ] * 10
        in1_data = [in1_value, ] * 10
        sink_data = in0_data

        tag = gr.tag_t()
        tag.key = pmt.to_pmt(tag_key)
        tag.value = pmt.to_pmt(tag_value)
        tag.offset = tag_offset

        # Only tag Input 1 of the share block and see if it transfers
        # to Output 0. Also verify that Input 0 stream is propagated to
        # Output 0.
        in0 = blocks.vector_source_c(in0_data, False, 1)
        in1 = blocks.vector_source_f(in1_data, False, 1, (tag,))
        tag_share = blocks.tag_share(gr.sizeof_gr_complex, gr.sizeof_float)
        sink = blocks.vector_sink_c(1)

        self.tb.connect(in0, (tag_share, 0))
        self.tb.connect(in1, (tag_share, 1))
        self.tb.connect(tag_share, sink)
        self.tb.run()

        self.assertEqual(len(sink.tags()), 1)
        # print(sink.tags())
        received_tag = sink.tags()[0]
        self.assertEqual(pmt.to_python(received_tag.key), tag_key)
        self.assertEqual(pmt.to_python(received_tag.value), tag_value)
        self.assertEqual(received_tag.offset, tag_offset)
        self.assertEqual(sink.data(), sink_data)


if __name__ == '__main__':
    gr_unittest.run(qa_tag_share)



#!/usr/bin/env python
#
# Copyright 2015,2016 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import time
import struct

import pmt
from gnuradio import gr, gr_unittest, digital, blocks
from gnuradio.digital import packet_utils


class test_packet_format_fb(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_packet_format_async_default(self):
        ac = packet_utils.default_access_code
        hdr_format = digital.header_format_default(ac, 0)

        formatter = digital.protocol_formatter_async(hdr_format)
        snk_hdr = blocks.message_debug()
        snk_pld = blocks.message_debug()

        self.tb.msg_connect(formatter, 'header', snk_hdr, 'store')
        self.tb.msg_connect(formatter, 'payload', snk_pld, 'store')

        send_str = b"Hello World"
        send_pmt = pmt.make_u8vector(len(send_str), 0)
        for i in range(len(send_str)):
            pmt.u8vector_set(send_pmt, i, send_str[i])
        msg = pmt.cons(pmt.PMT_NIL, send_pmt)

        port = pmt.intern("in")
        formatter.to_basic_block()._post(port, msg)

        self.tb.start()
        while (snk_hdr.num_messages() < 1) or (snk_pld.num_messages() < 1):
            time.sleep(0.1)

        self.tb.stop()
        self.tb.wait()

        result_hdr_pmt = pmt.cdr(snk_hdr.get_message(0))
        result_pld_pmt = pmt.cdr(snk_pld.get_message(0))

        result_hdr = pmt.u8vector_elements(result_hdr_pmt)
        result_pld = pmt.u8vector_elements(result_pld_pmt)
        header = bytes(result_hdr)
        payload = bytes(result_pld)

        access_code = packet_utils.default_access_code_binary
        rx_access_code = header[0:len(access_code)]

        length = len(send_str)
        rx_length = struct.unpack_from(b"!H", header, len(access_code))[0]

        self.assertEqual(access_code, rx_access_code)
        self.assertEqual(length, rx_length)
        self.assertEqual(length, len(payload))
        self.assertEqual(send_str, payload[0:length])

    def test_packet_parse_default(self):
        ac = packet_utils.default_access_code
        length = '0000000000000001'

        hdr_format_1bps = digital.header_format_default(ac, 0)
        hdr_format_4bps = digital.header_format_default(ac, 0, 4)

        ac_bits = [int(x) & 1 for x in ac]
        length_bits = [int(x) & 1 for x in length]
        header_bits = ac_bits + length_bits + length_bits

        src_hdr = blocks.vector_source_b(header_bits)

        parser_1bps = digital.protocol_parser_b(hdr_format_1bps)
        parser_4bps = digital.protocol_parser_b(hdr_format_4bps)

        snk_hdr_1bps = blocks.message_debug()
        snk_hdr_4bps = blocks.message_debug()

        self.tb.connect(src_hdr, parser_1bps)
        self.tb.connect(src_hdr, parser_4bps)

        self.tb.msg_connect(parser_1bps, 'info', snk_hdr_1bps, 'store')
        self.tb.msg_connect(parser_4bps, 'info', snk_hdr_4bps, 'store')

        self.tb.start()
        while (
                snk_hdr_1bps.num_messages() < 1) or (
                snk_hdr_4bps.num_messages() < 1):
            time.sleep(0.1)
        self.tb.stop()
        self.tb.wait()

        result_1bps = snk_hdr_1bps.get_message(0)
        result_4bps = snk_hdr_4bps.get_message(0)

        self.assertTrue(pmt.dict_has_key(
            result_1bps, pmt.intern('payload symbols')))
        self.assertEqual(pmt.to_long(pmt.dict_ref(
            result_1bps, pmt.intern('payload symbols'), pmt.PMT_F)), 8)

        self.assertTrue(pmt.dict_has_key(
            result_4bps, pmt.intern('payload symbols')))
        self.assertEqual(pmt.to_long(pmt.dict_ref(
            result_4bps, pmt.intern('payload symbols'), pmt.PMT_F)), 2)

    def test_packet_format_async_counter(self):
        bps = 2
        ac = packet_utils.default_access_code
        hdr_format = digital.header_format_counter(ac, 0, 2)

        formatter = digital.protocol_formatter_async(hdr_format)
        snk_hdr = blocks.message_debug()
        snk_pld = blocks.message_debug()

        self.tb.msg_connect(formatter, 'header', snk_hdr, 'store')
        self.tb.msg_connect(formatter, 'payload', snk_pld, 'store')

        send_str = b"Hello World" + 1000 * b"xxx"
        send_pmt = pmt.make_u8vector(len(send_str), 0)
        for i in range(len(send_str)):
            pmt.u8vector_set(send_pmt, i, send_str[i])
        msg = pmt.cons(pmt.PMT_NIL, send_pmt)

        port = pmt.intern("in")
        formatter.to_basic_block()._post(port, msg)

        self.tb.start()
        while (snk_hdr.num_messages() < 1) or (snk_pld.num_messages() < 1):
            time.sleep(0.1)
        self.tb.stop()
        self.tb.wait()

        result_hdr_pmt = pmt.cdr(snk_hdr.get_message(0))
        result_pld_pmt = pmt.cdr(snk_pld.get_message(0))

        result_hdr = pmt.u8vector_elements(result_hdr_pmt)
        result_pld = pmt.u8vector_elements(result_pld_pmt)
        header = bytes(result_hdr)
        payload = bytes(result_pld)

        access_code = packet_utils.default_access_code_binary
        rx_access_code = header[0:len(access_code)]

        length = len(send_str)
        rx_length = struct.unpack_from(b"!H", header, len(access_code))[0]
        rx_bps = struct.unpack_from(b"!H", header, len(access_code) + 4)[0]
        rx_counter = struct.unpack_from(b"!H", header, len(access_code) + 6)[0]

        self.assertEqual(access_code, rx_access_code)
        self.assertEqual(length, rx_length)
        self.assertEqual(bps, rx_bps)
        self.assertEqual(0, rx_counter)
        self.assertEqual(length, len(payload))
        self.assertEqual(send_str, payload[0:length])

    def test_packet_format_ofdm(self):
        """
        Test the header_format_ofdm object with scrambling
        """
        bphs = 8  # Header bps
        bpps = 8  # Payload bps
        scrambling = True
        occupied_carriers = [list(range(0, 40))]
        len_key_name = "packet_len"
        frame_key_name = "frame_len"
        num_key_name = "packet_num"
        hdr_format = digital.header_format_ofdm(
            occupied_carriers,
            1,
            len_key_name,
            frame_key_name,
            num_key_name,
            bphs,
            bpps,
            scrambling,
        )

        formatter = digital.protocol_formatter_bb(hdr_format, len_key_name)
        parser = digital.protocol_parser_b(hdr_format)

        send_str = b"Hello World" + 100 * b"xxx"
        send_bits = list(send_str)
        src = blocks.vector_source_b(
            send_bits + send_bits,
            repeat=False,
            tags=[
                gr.python_to_tag(
                    [
                        0,
                        pmt.intern(len_key_name),
                        pmt.to_pmt(len(send_bits)),
                        pmt.intern("vector_source"),
                    ]
                ),
                gr.python_to_tag(
                    [
                        len(send_bits),
                        pmt.intern(len_key_name),
                        pmt.to_pmt(len(send_bits)),
                        pmt.intern("vector_source"),
                    ]
                ),
            ],
        )

        repack = blocks.repack_bits_bb(8, 1, len_key_name, False, gr.GR_LSB_FIRST)
        snk_hdr = blocks.message_debug()

        self.tb.connect(src, formatter)
        self.tb.connect(formatter, repack)
        self.tb.connect(repack, parser)

        self.tb.msg_connect(parser, "info", snk_hdr, "store")
        self.tb.msg_connect(parser, "info", snk_hdr, "print")

        self.tb.start()
        while snk_hdr.num_messages() < 2:
            time.sleep(0.1)
        self.tb.stop()
        self.tb.wait()

        result_hdr_pmt = snk_hdr.get_message(0)

        self.assertTrue(pmt.dict_has_key(result_hdr_pmt, pmt.intern(frame_key_name)))
        self.assertEqual(
            pmt.to_long(
                pmt.dict_ref(result_hdr_pmt, pmt.intern(frame_key_name), pmt.PMT_F)
            ),
            len(send_str) // len(occupied_carriers[0]) + 1,
        )
        self.assertTrue(pmt.dict_has_key(result_hdr_pmt, pmt.intern(len_key_name)))
        self.assertEqual(
            pmt.to_long(
                pmt.dict_ref(result_hdr_pmt, pmt.intern(len_key_name), pmt.PMT_F)
            ),
            len(send_str),
        )
        self.assertTrue(pmt.dict_has_key(result_hdr_pmt, pmt.intern(num_key_name)))
        self.assertEqual(
            pmt.to_long(
                pmt.dict_ref(result_hdr_pmt, pmt.intern(num_key_name), pmt.PMT_F)
            ),
            0,
        )

        result_hdr_pmt = snk_hdr.get_message(1)

        self.assertTrue(pmt.dict_has_key(result_hdr_pmt, pmt.intern(frame_key_name)))
        self.assertEqual(
            pmt.to_long(
                pmt.dict_ref(result_hdr_pmt, pmt.intern(frame_key_name), pmt.PMT_F)
            ),
            len(send_str) // len(occupied_carriers[0]) + 1,
        )
        self.assertTrue(pmt.dict_has_key(result_hdr_pmt, pmt.intern(len_key_name)))
        self.assertEqual(
            pmt.to_long(
                pmt.dict_ref(result_hdr_pmt, pmt.intern(len_key_name), pmt.PMT_F)
            ),
            len(send_str),
        )
        self.assertTrue(pmt.dict_has_key(result_hdr_pmt, pmt.intern(num_key_name)))
        self.assertEqual(
            pmt.to_long(
                pmt.dict_ref(result_hdr_pmt, pmt.intern(num_key_name), pmt.PMT_F)
            ),
            1,
        )


if __name__ == '__main__':
    gr_unittest.run(test_packet_format_fb)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'usrp_display_qtgui.ui'
#
# Created: Thu Jul 16 22:06:24 2009
#      by: PyQt4 UI code generator 4.4.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(820, 774)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(240, 150))
        self.groupBox.setMaximumSize(QtCore.QSize(240, 16777215))
        self.groupBox.setObjectName("groupBox")
        self.formLayoutWidget = QtGui.QWidget(self.groupBox)
        self.formLayoutWidget.setGeometry(QtCore.QRect(10, 20, 221, 124))
        self.formLayoutWidget.setObjectName("formLayoutWidget")
        self.formLayout = QtGui.QFormLayout(self.formLayoutWidget)
        self.formLayout.setObjectName("formLayout")
        self.frequencyLabel = QtGui.QLabel(self.formLayoutWidget)
        self.frequencyLabel.setObjectName("frequencyLabel")
        self.formLayout.setWidget(
            0, QtGui.QFormLayout.LabelRole, self.frequencyLabel)
        self.gainLabel = QtGui.QLabel(self.formLayoutWidget)
        self.gainLabel.setObjectName("gainLabel")
        self.formLayout.setWidget(
            1, QtGui.QFormLayout.LabelRole, self.gainLabel)
        self.bandwidthLabel = QtGui.QLabel(self.formLayoutWidget)
        self.bandwidthLabel.setObjectName("bandwidthLabel")
        self.formLayout.setWidget(
            2, QtGui.QFormLayout.LabelRole, self.bandwidthLabel)
        self.frequencyEdit = QtGui.QLineEdit(self.formLayoutWidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.frequencyEdit.sizePolicy().hasHeightForWidth())
        self.frequencyEdit.setSizePolicy(sizePolicy)
        self.frequencyEdit.setMinimumSize(QtCore.QSize(120, 26))
        self.frequencyEdit.setObjectName("frequencyEdit")
        self.formLayout.setWidget(
            0, QtGui.QFormLayout.FieldRole, self.frequencyEdit)
        self.gainEdit = QtGui.QLineEdit(self.formLayoutWidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.gainEdit.sizePolicy().hasHeightForWidth())
        self.gainEdit.setSizePolicy(sizePolicy)
        self.gainEdit.setMinimumSize(QtCore.QSize(120, 26))
        self.gainEdit.setObjectName("gainEdit")
        self.formLayout.setWidget(
            1, QtGui.QFormLayout.FieldRole, self.gainEdit)
        self.bandwidthEdit = QtGui.QLineEdit(self.formLayoutWidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.bandwidthEdit.sizePolicy().hasHeightForWidth())
        self.bandwidthEdit.setSizePolicy(sizePolicy)
        self.bandwidthEdit.setMinimumSize(QtCore.QSize(120, 26))
        self.bandwidthEdit.setObjectName("bandwidthEdit")
        self.formLayout.setWidget(
            2, QtGui.QFormLayout.FieldRole, self.bandwidthEdit)
        self.amplifierLabel = QtGui.QLabel(self.formLayoutWidget)
        self.amplifierLabel.setObjectName("amplifierLabel")
        self.formLayout.setWidget(
            3, QtGui.QFormLayout.LabelRole, self.amplifierLabel)
        self.amplifierEdit = QtGui.QLineEdit(self.formLayoutWidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.amplifierEdit.sizePolicy().hasHeightForWidth())
        self.amplifierEdit.setSizePolicy(sizePolicy)
        self.amplifierEdit.setMinimumSize(QtCore.QSize(120, 26))
        self.amplifierEdit.setObjectName("amplifierEdit")
        self.formLayout.setWidget(
            3, QtGui.QFormLayout.FieldRole, self.amplifierEdit)
        self.horizontalLayout_2.addWidget(self.groupBox)
        self.frame_2 = QtGui.QFrame(self.centralwidget)
        self.frame_2.setMinimumSize(QtCore.QSize(200, 0))
        self.frame_2.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayoutWidget = QtGui.QWidget(self.frame_2)
        self.verticalLayoutWidget.setGeometry(QtCore.QRect(10, -1, 191, 151))
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.dcCancelCheckBox = QtGui.QCheckBox(self.verticalLayoutWidget)
        self.dcCancelCheckBox.setObjectName("dcCancelCheckBox")
        self.verticalLayout_3.addWidget(self.dcCancelCheckBox)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.dcGainLabel = QtGui.QLabel(self.verticalLayoutWidget)
        self.dcGainLabel.setObjectName("dcGainLabel")
        self.horizontalLayout.addWidget(self.dcGainLabel)
        self.dcGainEdit = QtGui.QLineEdit(self.verticalLayoutWidget)
        self.dcGainEdit.setObjectName("dcGainEdit")
        self.horizontalLayout.addWidget(self.dcGainEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        spacerItem = QtGui.QSpacerItem(
            20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.horizontalLayout_2.addWidget(self.frame_2)
        spacerItem1 = QtGui.QSpacerItem(
            40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem2 = QtGui.QSpacerItem(
            20, 80, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem2)
        self.pauseButton = QtGui.QPushButton(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pauseButton.sizePolicy().hasHeightForWidth())
        self.pauseButton.setSizePolicy(sizePolicy)
        self.pauseButton.setObjectName("pauseButton")
        self.verticalLayout.addWidget(self.pauseButton)
        self.closeButton = QtGui.QPushButton(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.closeButton.sizePolicy().hasHeightForWidth())
        self.closeButton.setSizePolicy(sizePolicy)
        self.closeButton.setMinimumSize(QtCore.QSize(75, 0))
        self.closeButton.setObjectName("closeButton")
        self.verticalLayout.addWidget(self.closeButton)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.gridLayout_2.addLayout(self.horizontalLayout_2, 1, 0, 1, 1)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.frame = QtGui.QFrame(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(
            QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(
            self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setMinimumSize(QtCore.QSize(800, 550))
        self.frame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtGui.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout = QtGui.QGridLayout(self.frame)
        self.gridLayout.setObjectName("gridLayout")
        self.sinkLayout = QtGui.QHBoxLayout()
        self.sinkLayout.setObjectName("sinkLayout")
        self.gridLayout.addLayout(self.sinkLayout, 0, 0, 1, 1)
        self.verticalLayout_2.addWidget(self.frame)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 820, 24))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionExit = QtGui.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionSaveData = QtGui.QAction(MainWindow)
        self.actionSaveData.setObjectName("actionSaveData")
        self.menuFile.addAction(self.actionSaveData)
        self.menuFile.addAction(self.actionExit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QObject.connect(self.closeButton, QtCore.SIGNAL(
            "clicked()"), MainWindow.close)
        QtCore.QObject.connect(self.actionExit, QtCore.SIGNAL(
            "triggered()"), MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate(
            "MainWindow", "USRP Display", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate(
            "MainWindow", "Receiver Parameters", None, QtGui.QApplication.UnicodeUTF8))
        self.frequencyLabel.setText(QtGui.QApplication.translate(
            "MainWindow", "Frequency (Hz)", None, QtGui.QApplication.UnicodeUTF8))
        self.gainLabel.setText(QtGui.QApplication.translate(
            "MainWindow", "RF Gain", None, QtGui.QApplication.UnicodeUTF8))
        self.bandwidthLabel.setText(QtGui.QApplication.translate(
            "MainWindow", "Bandwidth", None, QtGui.QApplication.UnicodeUTF8))
        self.amplifierLabel.setText(QtGui.QApplication.translate(
            "MainWindow", "Amplifier", None, QtGui.QApplication.UnicodeUTF8))
        self.dcCancelCheckBox.setText(QtGui.QApplication.translate(
            "MainWindow", "Cancel DC", None, QtGui.QApplication.UnicodeUTF8))
        self.dcGainLabel.setText(QtGui.QApplication.translate(
            "MainWindow", "DC Canceller Gain", None, QtGui.QApplication.UnicodeUTF8))
        self.pauseButton.setText(QtGui.QApplication.translate(
            "MainWindow", "Pause", None, QtGui.QApplication.UnicodeUTF8))
        self.closeButton.setText(QtGui.QApplication.translate(
            "MainWindow", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate(
            "MainWindow", "&File", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setText(QtGui.QApplication.translate(
            "MainWindow", "E&xit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSaveData.setText(QtGui.QApplication.translate(
            "MainWindow", "&Save Data", None, QtGui.QApplication.UnicodeUTF8))



#!/usr/bin/env python
#
# Copyright 2016 Free Software Foundation, Inc.
# Copyright 2021 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest


class test_flowgraph (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000_empty_fg(self):
        self.tb.start()
        self.tb.stop()


if __name__ == '__main__':
    gr_unittest.run(test_flowgraph)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, pdu
from gnuradio import network
import random
import pmt


class qa_socket_pdu_stop (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_stop_streaming(self):
        # Test that block stops when interacting with streaming interface
        # port = str(random.Random().randint(0, 30000) + 10000)
        srcdata = (
            0x73,
            0x75,
            0x63,
            0x68,
            0x74,
            0x65,
            0x73,
            0x74,
            0x76,
            0x65,
            0x72,
            0x79,
            0x70,
            0x61,
            0x73,
            0x73)
        tag_dict = {"offset": 0,
                    "key": pmt.intern("len"),
                    "value": pmt.from_long(8)}
        tag1 = gr.python_to_tag(tag_dict)
        tag_dict["offset"] = 8
        tag2 = gr.python_to_tag(tag_dict)
        tags = [tag1, tag2]

        src = blocks.vector_source_b(srcdata, False, 1, tags)
        ts_to_pdu = pdu.tagged_stream_to_pdu(gr.types.byte_t, "len")
        pdu_send = network.socket_pdu("UDP_CLIENT", "localhost", "4141")
        # pdu_recv = network.socket_pdu("UDP_SERVER", "localhost", port)
        # pdu_to_ts = pdu.pdu_to_tagged_stream(gr.types.byte_t, "len")
        # head = blocks.head(gr.sizeof_char, 10)
        # sink = blocks.vector_sink_b(1)

        self.tb.connect(src, ts_to_pdu)
        self.tb.msg_connect(ts_to_pdu, "pdus", pdu_send, "pdus")
        # a UDP socket connects pdu_send to pdu_recv
        # TODO: test that the recv socket can be destroyed from downstream
        # that signals DONE. Also that we get the PDUs we sent
        # self.tb.msg_connect(pdu_recv, "pdus", pdu_to_ts, "pdus")
        # self.tb.connect(pdu_to_ts, head, sink)
        self.tb.run()


if __name__ == '__main__':
    gr_unittest.run(qa_socket_pdu_stop)



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from thrift import Thrift
from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol
from gnuradio.ctrlport.GNURadio import ControlPort
from gnuradio.ctrlport import RPCConnection
from gnuradio import gr
import pmt
import sys


class ThriftRadioClient(object):
    def __init__(self, host, port):
        self.tsocket = TSocket.TSocket(host, port)
        self.transport = TTransport.TBufferedTransport(self.tsocket)
        self.protocol = TBinaryProtocol.TBinaryProtocol(self.transport)

        self.radio = ControlPort.Client(self.protocol)
        self.transport.open()
        self.host = host
        self.port = port

    def __del__(self):
        try:
            self.transport.close()
            self.radio.shutdown()
        except:
            pass

    def getRadio(self):
        return self.radio


"""
RPC Client interface for the Apache Thrift middle-ware RPC transport.

Args:
    port: port number of the connection
    host: hostname of the connection
"""


class RPCConnectionThrift(RPCConnection.RPCConnection):
    class Knob(object):
        def __init__(self, key, value=None, ktype=0):
            (self.key, self.value, self.ktype) = (key, value, ktype)

        def __repr__(self):
            return "({0} = {1})".format(self.key, self.value)

    def __init__(self, host=None, port=None):
        from gnuradio.ctrlport.GNURadio import ttypes
        self.BaseTypes = ttypes.BaseTypes
        self.KnobBase = ttypes.KnobBase

        # If not set by the user, get the port number from the thrift
        # config file, if one is set. Defaults to 9090 otherwise.
        if port is None:
            p = gr.prefs()
            thrift_config_file = p.get_string("ControlPort", "config", "")
            if(len(thrift_config_file) > 0):
                p.add_config_file(thrift_config_file)
                port = p.get_long("thrift", "port", 9090)
            else:
                port = 9090
        else:
            port = int(port)

        super(RPCConnectionThrift, self).__init__(
            method='thrift', port=port, host=host)
        self.newConnection(host, port)

        self.unpack_dict = {
            self.BaseTypes.BOOL: lambda k, b: self.Knob(k, b.value.a_bool, self.BaseTypes.BOOL),
            self.BaseTypes.BYTE: lambda k, b: self.Knob(k, b.value.a_byte, self.BaseTypes.BYTE),
            self.BaseTypes.SHORT: lambda k, b: self.Knob(k, b.value.a_short, self.BaseTypes.SHORT),
            self.BaseTypes.INT: lambda k, b: self.Knob(k, b.value.a_int, self.BaseTypes.INT),
            self.BaseTypes.LONG: lambda k, b: self.Knob(k, b.value.a_long, self.BaseTypes.LONG),
            self.BaseTypes.DOUBLE: lambda k, b: self.Knob(k, b.value.a_double, self.BaseTypes.DOUBLE),
            self.BaseTypes.STRING: lambda k, b: self.Knob(k, b.value.a_string, self.BaseTypes.STRING),
            self.BaseTypes.COMPLEX: lambda k, b: self.Knob(k, b.value.a_complex, self.BaseTypes.COMPLEX),
            self.BaseTypes.F32VECTOR: lambda k, b: self.Knob(k, b.value.a_f32vector, self.BaseTypes.F32VECTOR),
            self.BaseTypes.F64VECTOR: lambda k, b: self.Knob(k, b.value.a_f64vector, self.BaseTypes.F64VECTOR),
            self.BaseTypes.S64VECTOR: lambda k, b: self.Knob(k, b.value.a_s64vector, self.BaseTypes.S64VECTOR),
            self.BaseTypes.S32VECTOR: lambda k, b: self.Knob(k, b.value.a_s32vector, self.BaseTypes.S32VECTOR),
            self.BaseTypes.S16VECTOR: lambda k, b: self.Knob(k, b.value.a_s16vector, self.BaseTypes.S16VECTOR),
            self.BaseTypes.S8VECTOR: lambda k, b: self.Knob(k, b.value.a_s8vector, self.BaseTypes.S8VECTOR),
            self.BaseTypes.C32VECTOR: lambda k, b: self.Knob(k, b.value.a_c32vector, self.BaseTypes.C32VECTOR),
        }

        self.pack_dict = {
            self.BaseTypes.BOOL: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_bool=k.value)),
            self.BaseTypes.BYTE: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_byte=k.value)),
            self.BaseTypes.SHORT: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_short=k.value)),
            self.BaseTypes.INT: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_int=k.value)),
            self.BaseTypes.LONG: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_long=k.value)),
            self.BaseTypes.DOUBLE: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_double=k.value)),
            self.BaseTypes.STRING: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_string=k.value)),
            self.BaseTypes.COMPLEX: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_complex=k.value)),
            self.BaseTypes.F32VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_f32vector=k.value)),
            self.BaseTypes.F64VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_f64vector=k.value)),
            self.BaseTypes.S64VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_s64vector=k.value)),
            self.BaseTypes.S32VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_s32vector=k.value)),
            self.BaseTypes.S16VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_s16vector=k.value)),
            self.BaseTypes.S8VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_s8vector=k.value)),
            self.BaseTypes.C32VECTOR: lambda k: ttypes.Knob(type=k.ktype, value=ttypes.KnobBase(a_c32vector=k.value)),
        }

    def __str__(self):
        return "Apache Thrift connection to {0}:{1}".format(
            self.thriftclient.host,
            self.thriftclient.port)

    def unpackKnob(self, key, knob):
        f = self.unpack_dict.get(knob.type, None)
        if(f):
            return f(key, knob)
        else:
            sys.stderr.write(
                "unpackKnobs: Incorrect Knob type: {0}\n".format(knob.type))
            raise exceptions.ValueError

    def packKnob(self, knob):
        f = self.pack_dict.get(knob.ktype, None)
        if(f):
            return f(knob)
        else:
            sys.stderr.write(
                "packKnobs: Incorrect Knob type: {0}\n".format(knob.type))
            raise exceptions.ValueError

    def newConnection(self, host=None, port=None):
        self.thriftclient = ThriftRadioClient(host, int(port))

    def properties(self, *args):
        knobprops = self.thriftclient.radio.properties(*args)
        for key, knobprop in list(knobprops.items()):
            #print("key:", key, "value:", knobprop, "type:", knobprop.type)
            knobprops[key].min = self.unpackKnob(key, knobprop.min)
            knobprops[key].max = self.unpackKnob(key, knobprop.max)
            knobprops[key].defaultvalue = self.unpackKnob(
                key, knobprop.defaultvalue)
        return knobprops

    def getKnobs(self, *args):
        result = {}
        for key, knob in list(self.thriftclient.radio.getKnobs(*args).items()):
            #print("key:", key, "value:", knob, "type:", knob.type)
            result[key] = self.unpackKnob(key, knob)

            # If complex, convert to Python complex
            # FIXME: better list iterator way to handle this?
            if(knob.type == self.BaseTypes.C32VECTOR):
                for i in range(len(result[key].value)):
                    result[key].value[i] = complex(result[key].value[i].re,
                                                   result[key].value[i].im)
        return result

    def getKnobsRaw(self, *args):
        result = {}
        for key, knob in list(self.thriftclient.radio.getKnobs(*args).items()):
            #print("key:", key, "value:", knob, "type:", knob.type)
            result[key] = knob
        return result

    def getRe(self, *args):
        result = {}
        for key, knob in list(self.thriftclient.radio.getRe(*args).items()):
            result[key] = self.unpackKnob(key, knob)
        return result

    def setKnobs(self, *args):
        if(type(*args) == dict):
            a = dict(*args)
            result = {}
            for key, knob in list(a.items()):
                result[key] = self.packKnob(knob)
            self.thriftclient.radio.setKnobs(result)
        elif(type(*args) == list or type(*args) == tuple):
            a = list(*args)
            result = {}
            for k in a:
                result[k.key] = self.packKnob(k)
            self.thriftclient.radio.setKnobs(result)
        else:
            sys.stderr.write(
                "setKnobs: Invalid type; must be dict, list, or tuple\n")

    def shutdown(self):
        self.thriftclient.radio.shutdown()

    def postMessage(self, blk_alias, port, msg):
        '''
        blk_alias: the alias of the block we are posting the message
                   to; must have an open message port named 'port'.
                   Provide as a string.
        port: The name of the message port we are sending the message to.
              Provide as a string.
        msg: The actual message. Provide this as a PMT of the form
             right for the message port.
        The alias and port names are converted to PMT symbols and
        serialized. The msg is already a PMT and so just serialized.
        '''
        self.thriftclient.radio.postMessage(pmt.serialize_str(pmt.intern(blk_alias)),
                                            pmt.serialize_str(
                                                pmt.intern(port)),
                                            pmt.serialize_str(msg))

    def printProperties(self, props):
        info = ""
        info += "Item:\t\t{0}\n".format(props.description)
        info += "units:\t\t{0}\n".format(props.units)
        info += "min:\t\t{0}\n".format(props.min.value)
        info += "max:\t\t{0}\n".format(props.max.value)
        info += "default:\t\t{0}\n".format(props.defaultvalue.value)
        info += "Type Code:\t0x{0:x}\n".format(props.type)
        info += "Disp Code:\t0x{0:x}\n".format(props.display)
        return info



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_head(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_head(self):
        src_data = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        expected_result = [1, 2, 3, 4]
        src1 = blocks.vector_source_i(src_data)
        op = blocks.head(gr.sizeof_int, 4)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op)
        self.tb.connect(op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)


if __name__ == '__main__':
    gr_unittest.run(test_head)



#
# Copyright 2010 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""
Utilities for extracting text from generated classes.
"""


def is_string(txt):
    if isinstance(txt, str):
        return True
    try:
        if isinstance(txt, unicode):
            return True
    except NameError:
        pass
    return False


def description(obj):
    if obj is None:
        return None
    return description_bit(obj).strip()


def description_bit(obj):
    if hasattr(obj, 'content'):
        contents = [description_bit(item) for item in obj.content]
        result = ''.join(contents)
    elif hasattr(obj, 'content_'):
        contents = [description_bit(item) for item in obj.content_]
        result = ''.join(contents)
    elif hasattr(obj, 'value'):
        result = description_bit(obj.value)
    elif is_string(obj):
        return obj
    else:
        raise Exception(
            'Expecting a string or something with content, content_ or value attribute')
    # If this bit is a paragraph then add one some line breaks.
    if hasattr(obj, 'name') and obj.name == 'para':
        result += "\n\n"
    return result



#!/usr/bin/env python

import math
import re

EVERYTHING_BEFORE_PARAMS = """id: qtgui_time_sink_x
label: QT GUI Time Sink

parameters:
-   id: type
    label: Type
    dtype: enum
    default: complex
    options: [complex, float, msg_complex, msg_float]
    option_labels: [Complex, Float, Complex Message, Float Message]
    option_attributes:
        fcn: [time_sink_c, time_sink_f, time_sink_c, time_sink_f]
        t: [complex, float, message, message]
    hide: part
-   id: name
    label: Name
    dtype: string
    default: '""'
    hide: ${ ('none' if len(name) > 0 else 'part') }
-   id: ylabel
    label: Y Axis Label
    dtype: string
    default: Amplitude
    hide: part
-   id: yunit
    label: Y Axis Unit
    dtype: string
    default: '""'
    hide: part
-   id: size
    label: Number of Points
    dtype: int
    default: '1024'
    hide: ${ ('all' if type.startswith('msg') else 'none') }
-   id: srate
    label: Sample Rate
    dtype: float
    default: samp_rate
-   id: grid
    label: Grid
    dtype: enum
    default: 'False'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
    hide: part
-   id: autoscale
    label: Autoscale
    dtype: enum
    default: 'False'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
-   id: ymin
    label: Y min
    dtype: float
    default: '-1'
    hide: part
-   id: ymax
    label: Y max
    dtype: float
    default: '1'
    hide: part
-   id: nconnections
    label: Number of Inputs
    dtype: int
    default: '1'
    hide: ${ ('all' if type.startswith('msg') else 'part') }
-   id: update_time
    label: Update Period
    dtype: float
    default: '0.10'
    hide: part
-   id: entags
    label: Disp. Tags
    dtype: enum
    default: 'True'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
    hide: ${ ('all' if type.startswith('msg') else 'part') }
-   id: gui_hint
    label: GUI Hint
    dtype: gui_hint
    hide: part
-   id: tr_mode
    label: Trigger Mode
    category: Trigger
    dtype: enum
    default: qtgui.TRIG_MODE_FREE
    options: [qtgui.TRIG_MODE_FREE, qtgui.TRIG_MODE_AUTO, qtgui.TRIG_MODE_NORM, qtgui.TRIG_MODE_TAG]
    option_labels: [Free, Auto, Normal, Tag]
    hide: part
-   id: tr_slope
    label: Trigger Slope
    category: Trigger
    dtype: enum
    default: qtgui.TRIG_MODE_POS
    options: [qtgui.TRIG_SLOPE_POS, qtgui.TRIG_SLOPE_NEG]
    option_labels: [Positive, Negative]
    hide: part
-   id: tr_level
    label: Trigger Level
    category: Trigger
    dtype: float
    default: '0.0'
    hide: part
-   id: tr_delay
    label: Trigger Delay
    category: Trigger
    dtype: float
    default: '0'
    hide: part
-   id: tr_chan
    label: Trigger Channel
    category: Trigger
    dtype: int
    default: '0'
    hide: part
-   id: tr_tag
    label: Trigger Tag Key
    category: Trigger
    dtype: string
    default: '""'
    hide: part
-   id: ctrlpanel
    label: Control Panel
    category: Config
    dtype: enum
    default: 'False'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
    hide: part
-   id: legend
    label: Legend
    category: Config
    dtype: enum
    default: 'True'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
    hide: part
-   id: axislabels
    label: Axis Labels
    category: Config
    dtype: enum
    default: 'True'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
    hide: part
-   id: stemplot
    label: Stem Plot
    category: Config
    dtype: enum
    default: 'False'
    options: ['True', 'False']
    option_labels: ['Yes', 'No']
    hide: part"""

LINE_PARAMS = """
-   id: label{i}
    label: Line {i} Label
    dtype: string
    default: 'Signal {i}'
    base_key: label1
    hide: ${{ ('part' if (
            int(nconnections) >= {i}
            or (type == "complex" and int(nconnections) >= {i_cplx})
            or (type == "msg_complex" and {i_cplx} <= 1)
            or (type == "msg_float" and {i} <= 1))
        else 'all')
        }}
    category: Config
-   id: width{i}
    label: Line {i} Width
    default: 1
    base_key: width1
    hide: ${{ ('part' if (
            int(nconnections) >= {i}
            or (type == "complex" and int(nconnections) >= {i_cplx})
            or (type == "msg_complex" and {i_cplx} <= 1)
            or (type == "msg_float" and {i} <= 1))
        else 'all')
        }}
    category: Config
-   id: color{i}
    label: Line {i} Color
    dtype: enum
    options: ['blue', 'red', 'green', 'black', 'cyan', 'magenta', 'yellow', 'dark red', 'dark green', 'dark blue']
    option_labels: ['Blue', 'Red', 'Green', 'Black', 'Cyan', 'Magenta', 'Yellow', 'Dark Red', 'Dark Green', 'Dark Blue']
    default: '{i_color}'
    base_key: color1
    hide: ${{ ('part' if (
            int(nconnections) >= {i}
            or (type == "complex" and int(nconnections) >= {i_cplx})
            or (type == "msg_complex" and {i_cplx} <= 1)
            or (type == "msg_float" and {i} <= 1))
        else 'all')
        }}
    category: Config
-   id: style{i}
    label: Line {i} Style
    dtype: enum
    options: ['1','2','3','4','5','0']
    option_labels: ['Solid','Dash','Dots','Dash-Dot','Dash-Dot-Dot']
    default: 1
    base_key: style1
    hide: ${{ ('part' if (
            int(nconnections) >= {i}
            or (type == "complex" and int(nconnections) >= {i_cplx})
            or (type == "msg_complex" and {i_cplx} <= 1)
            or (type == "msg_float" and {i} <= 1))
        else 'all')
        }}
    category: Config
-   id: marker{i}
    label: Line {i} Marker
    dtype: enum
    options: ['-1','0','1','2','3','4','5','6','7','8','9']
    option_labels: ['None','Circle','Rectangle','Diamond','Triangle','Down Triangle','Left Triangle','Right Triangle','Cross','X-Cross']
    default: -1
    base_key: marker1
    hide: ${{ ('part' if (
            int(nconnections) >= {i}
            or (type == "complex" and int(nconnections) >= {i_cplx})
            or (type == "msg_complex" and {i_cplx} <= 1)
            or (type == "msg_float" and {i} <= 1))
        else 'all')
        }}
    category: Config
-   id: alpha{i}
    label: Line {i} Alpha
    dtype: real
    default: 1.0
    base_key: alpha1
    hide: ${{ ('part' if (
            int(nconnections) >= {i}
            or (type == "complex" and int(nconnections) >= {i_cplx})
            or (type == "msg_complex" and {i_cplx} <= 1)
            or (type == "msg_float" and {i} <= 1))
        else 'all')
        }}
    category: Config
"""

EVERYTHING_AFTER_PARAMS = """
asserts:
- ${nconnections <= (5 if type == 'complex' else 10)}

inputs:
-   domain: stream
    dtype: ${ type.t }
    multiplicity: ${ (0 if type.startswith('msg') else nconnections) }
    optional: ${ (True if type.startswith('msg') else False) }

templates:
    imports: |-
        import sip
    
    callbacks:
    - set_time_domain_axis(${min}, ${max})
    - set_update_time(${update_time})
    - set_y_axis(${ymin}, ${ymax})
    - set_samp_rate(${srate})
    - self.${id}.set_trigger_mode(${tr_mode}, ${tr_slope}, ${tr_level}, ${tr_delay},
        ${tr_chan}, ${tr_tag})
    make: |-
        <%
            win = 'self._%s_win'%id
        %>\\
        qtgui.${type.fcn}(
            ${size}, #size
            ${srate}, #samp_rate
            ${name}, #name
            ${0 if type.startswith('msg') else nconnections}, #number of inputs
            None # parent
        )
        self.${id}.set_update_time(${update_time})
        self.${id}.set_y_axis(${ymin}, ${ymax})

        self.${id}.set_y_label(${ylabel}, ${yunit})

        self.${id}.enable_tags(${entags})
        self.${id}.set_trigger_mode(${tr_mode}, ${tr_slope}, ${tr_level}, ${tr_delay}, ${tr_chan}, ${tr_tag})
        self.${id}.enable_autoscale(${autoscale})
        self.${id}.enable_grid(${grid})
        self.${id}.enable_axis_labels(${axislabels})
        self.${id}.enable_control_panel(${ctrlpanel})
        self.${id}.enable_stem_plot(${stemplot})

        % if legend == "False":
        self.${id}.disable_legend()
        % endif

        labels = [${label1}, ${label2}, ${label3}, ${label4}, ${label5},
            ${label6}, ${label7}, ${label8}, ${label9}, ${label10}]
        widths = [${width1}, ${width2}, ${width3}, ${width4}, ${width5},
            ${width6}, ${width7}, ${width8}, ${width9}, ${width10}]
        colors = ['${color1}', '${color2}', '${color3}', '${color4}', '${color5}',
            '${color6}', '${color7}', '${color8}', '${color9}', '${color10}']
        alphas = [${alpha1}, ${alpha2}, ${alpha3}, ${alpha4}, ${alpha5},
            ${alpha6}, ${alpha7}, ${alpha8}, ${alpha9}, ${alpha10}]
        styles = [${style1}, ${style2}, ${style3}, ${style4}, ${style5},
            ${style6}, ${style7}, ${style8}, ${style9}, ${style10}]
        markers = [${marker1}, ${marker2}, ${marker3}, ${marker4}, ${marker5},
            ${marker6}, ${marker7}, ${marker8}, ${marker9}, ${marker10}]


        % if type.endswith('complex'):
        for i in range(${2 if type.startswith('msg') else 2*int(nconnections)}):
            if len(labels[i]) == 0:
                if (i % 2 == 0):
                    self.${id}.set_line_label(i, "Re{{Data {0}}}".format(i/2))
                else:
                    self.${id}.set_line_label(i, "Im{{Data {0}}}".format(i/2))
            else:
                self.${id}.set_line_label(i, labels[i])
            self.${id}.set_line_width(i, widths[i])
            self.${id}.set_line_color(i, colors[i])
            self.${id}.set_line_style(i, styles[i])
            self.${id}.set_line_marker(i, markers[i])
            self.${id}.set_line_alpha(i, alphas[i])
        % else:
        for i in range(${1 if type.startswith('msg') else int(nconnections)}):
            if len(labels[i]) == 0:
                self.${id}.set_line_label(i, "Data {0}".format(i))
            else:
                self.${id}.set_line_label(i, labels[i])
            self.${id}.set_line_width(i, widths[i])
            self.${id}.set_line_color(i, colors[i])
            self.${id}.set_line_style(i, styles[i])
            self.${id}.set_line_marker(i, markers[i])
            self.${id}.set_line_alpha(i, alphas[i])
        % endif

        ${win} = sip.wrapinstance(self.${id}.qwidget(), Qt.QWidget)
        ${gui_hint() % win}

documentation: |-
    The GUI hint can be used to position the widget within the application. The hint is of the form [tab_id@tab_index]: [row, col, row_span, col_span]. Both the tab specification and the grid position are optional.

file_format: 1
"""


def make_yml():
    """Return the YML file as a string"""
    default_colors = [
        'blue', 'red', 'green', 'black', 'cyan', 'magenta', 'yellow',
        'dark red', 'dark green', 'dark blue'
    ]
    line_params_1 = LINE_PARAMS.format(
        i=1, i_cplx=1, i_color=default_colors[0])
    line_params_1 = re.sub(r'    base_key:.*\n', '', line_params_1)
    line_params_n = ''.join([
        LINE_PARAMS.format(
            i=i,
            i_cplx=int(math.ceil(float(i) / 2)),
            i_color=default_colors[(i - 1) % len(default_colors)],
        )
        for i in range(2, 11)
    ])
    return ''.join((
        EVERYTHING_BEFORE_PARAMS,
        line_params_1,
        line_params_n,
        EVERYTHING_AFTER_PARAMS,
    ))


if __name__ == '__main__':
    import sys
    try:
        filename = sys.argv[1]
    except IndexError:
        filename = __file__[:-3]
    data = make_yml()
    with open(filename, 'wb') as fp:
        fp.write(data.encode())



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, gr_unittest, analog, blocks


class test_cpfsk_bc(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_cpfsk_bc_001(self):
        # Test set/gets

        op = analog.cpfsk_bc(2, 1, 2)

        op.set_amplitude(2)
        a = op.amplitude()
        self.assertEqual(2, a)

        freq = 2 * math.pi / 2.0
        f = op.freq()
        self.assertAlmostEqual(freq, f, 5)

        p = op.phase()
        self.assertEqual(0, p)

    def test_cpfsk_bc_002(self):
        src_data = 10 * [0, 1]
        expected_result = [complex(2 * x - 1, 0) for x in src_data]

        src = blocks.vector_source_b(src_data)
        op = analog.cpfsk_bc(2, 1, 2)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()[0:len(expected_result)]
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_cpfsk_bc)



#!/usr/bin/env python
#
# Copyright 2008,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy
from gnuradio import gr, gr_unittest, wavelet, analog, blocks
import copy
# import pygsl.wavelet as wavelet # FIXME: pygsl not checked for in config
import math


def sqr(x):
    return x * x


def np2(k):
    m = 0
    n = k - 1
    while n > 0:
        m += 1
    return m


class test_classify(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

#     def test_000_(self):
#         src_data = numpy.zeros(10)
#         trg_data = numpy.zeros(10)
#         src = blocks.vector_source_f(src_data)
#         dst = blocks.vector_sink_f()
#         self.tb.connect(src, dst)
#         self.tb.run()
#         rsl_data = dst.data()
#         sum = 0
#         for (u,v) in zip(trg_data, rsl_data):
#             w = u - v
#             sum += w * w
#         sum /= float(len(trg_data))
#         assert sum < 1e-6

    def test_001_(self):
        src_data = numpy.array([-1.0, 1.0, -1.0, 1.0])
        trg_data = src_data * 0.5
        src = blocks.vector_source_f(src_data)
        dst = blocks.vector_sink_f()
        rail = analog.rail_ff(-0.5, 0.5)
        self.tb.connect(src, rail)
        self.tb.connect(rail, dst)
        self.tb.run()
        rsl_data = dst.data()
        sum = 0
        for (u, v) in zip(trg_data, rsl_data):
            w = u - v
            sum += w * w
        sum /= float(len(trg_data))
        assert sum < 1e-6

    def test_002_(self):
        src_data = numpy.array([-1.0,
                                -1.0 / 2.0,
                                -1.0 / 3.0,
                                -1.0 / 4.0,
                                -1.0 / 5.0])
        trg_data = copy.deepcopy(src_data)

        src = blocks.vector_source_f(src_data, False, len(src_data))
        st = blocks.stretch_ff(-1.0 / 5.0, len(src_data))
        dst = blocks.vector_sink_f(len(src_data))
        self.tb.connect(src, st)
        self.tb.connect(st, dst)
        self.tb.run()
        rsl_data = dst.data()
        sum = 0
        for (u, v) in zip(trg_data, rsl_data):
            w = u - v
            sum += w * w
        sum /= float(len(trg_data))
        assert sum < 1e-6

    def test_003_(self):
        src_grid = (0.0, 1.0, 2.0, 3.0, 4.0)
        trg_grid = copy.deepcopy(src_grid)
        src_data = (0.0, 1.0, 0.0, 1.0, 0.0)

        src = blocks.vector_source_f(src_data, False, len(src_grid))
        sq = wavelet.squash_ff(src_grid, trg_grid)
        dst = blocks.vector_sink_f(len(trg_grid))
        self.tb.connect(src, sq)
        self.tb.connect(sq, dst)
        self.tb.run()
        rsl_data = dst.data()
        sum = 0
        for (u, v) in zip(src_data, rsl_data):
            w = u - v
            sum += w * w
        sum /= float(len(src_data))
        assert sum < 1e-6

#    def test_004_(self): # FIXME: requires pygsl
#
#        n = 256
#        o = 4
#        ws = wavelet.workspace(n)
#        w = wavelet.daubechies(o)
#
#        a = numpy.arange(n)
#        b = numpy.sin(a*numpy.pi/16.0)
#        c = w.transform_forward(b, ws)
#        d = w.transform_inverse(c, ws)
#
#        src = gr.vector_source_f(b, False, n)
#        wv = wavelet.wavelet_ff(n, o, True)
#        src = blocks.vector_source_f(b, False, n)
#        wv = wavelet.wavelet_ff(n, o, True)
#
#        dst = blocks.vector_sink_f(n)
#        self.tb.connect(src, wv)
#        self.tb.connect(wv, dst)
#        self.tb.run()
#        e = dst.data()
#
#        sum = 0
#        for (u, v) in zip(c, e):
#            w = u - v
#            sum += w * w
#        sum /= float(len(c))
#        assert sum < 1e-6

    def test_005_(self):

        src_data = (1.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0)

        dwav = numpy.array(src_data)
        wvps = numpy.zeros(3)
        # wavelet power spectrum
        scl = 1.0 / sqr(dwav[0])
        k = 1
        for e in range(len(wvps)):
            wvps[e] = scl * sqr(dwav[k:k + (0o1 << e)]).sum()
            k += 0o1 << e

        src = blocks.vector_source_f(src_data, False, len(src_data))
        kon = wavelet.wvps_ff(len(src_data))
        dst = blocks.vector_sink_f(int(math.ceil(math.log(len(src_data), 2))))

        self.tb.connect(src, kon)
        self.tb.connect(kon, dst)

        self.tb.run()
        snk_data = dst.data()

        sum = 0
        for (u, v) in zip(snk_data, wvps):
            w = u - v
            sum += w * w
        sum /= float(len(snk_data))
        assert sum < 1e-6


if __name__ == '__main__':
    gr_unittest.run(test_classify)



#!/usr/bin/env python
#
# Copyright 2012-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_tag_debug(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        # Just run some data through and make sure it doesn't puke.
        src_data = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
        src = blocks.vector_source_i(src_data)
        op = blocks.tag_debug(gr.sizeof_int, "tag QA")
        self.tb.connect(src, op)
        self.tb.run()
        x = op.current_tags()


if __name__ == '__main__':
    gr_unittest.run(test_tag_debug)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018-2021 National Technology & Engineering Solutions of Sandia, LLC
# (NTESS). Under the terms of Contract DE-NA0003525 with NTESS, the U.S. Government
# retains certain rights in this software.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, gr_unittest, blocks, pdu
import numpy as np
import pmt
import time


class qa_pdu_to_bursts (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.p2s = pdu.pdu_to_stream_c(pdu.EARLY_BURST_APPEND, 64)
        self.vs = blocks.vector_sink_c(1)
        self.tag_debug = blocks.tag_debug(gr.sizeof_gr_complex * 1, '', "")
        self.tag_debug.set_display(True)

        self.tb.connect((self.p2s, 0), (self.vs, 0))
        self.tb.connect((self.p2s, 0), (self.tag_debug, 0))

    def tearDown(self):
        self.tb = None

    def test_001_basic(self):
        in_data = [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
                   1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1]
        in_pdu = pmt.cons(
            pmt.make_dict(), pmt.init_c32vector(len(in_data), in_data))
        e_tag_0 = gr.tag_utils.python_to_tag(
            (0, pmt.intern("tx_sob"), pmt.PMT_T, pmt.PMT_NIL))
        e_tag_1 = gr.tag_utils.python_to_tag(
            (len(in_data) - 1, pmt.intern("tx_eob"), pmt.PMT_T, pmt.PMT_NIL))

        self.tb.start()
        self.p2s.to_basic_block()._post(pmt.intern("pdus"), pmt.intern("MALFORMED PDU"))
        self.p2s.to_basic_block()._post(pmt.intern("pdus"), in_pdu)
        self.waitFor(lambda: len(self.vs.tags()) == 2,
                     timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        tags = self.vs.tags()
        self.assertEqual(len(tags), 2)
        self.assertEqual(tags[0].offset, e_tag_0.offset)
        self.assertTrue(pmt.equal(tags[0].key, e_tag_0.key))
        self.assertTrue(pmt.equal(tags[0].value, e_tag_0.value))
        self.assertEqual(tags[1].offset, e_tag_1.offset)
        self.assertTrue(pmt.equal(tags[1].key, e_tag_1.key))
        self.assertTrue(pmt.equal(tags[1].value, e_tag_1.value))
        self.assertTrue((in_data == np.real(self.vs.data())).all())

    def test_002_timed(self):
        in_data = [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
                   1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1]
        tag_time = pmt.make_tuple(
            pmt.from_uint64(11), pmt.from_double(0.123456))
        in_dict = pmt.dict_add(
            pmt.make_dict(), pmt.intern("tx_time"), tag_time)
        in_pdu = pmt.cons(in_dict, pmt.init_c32vector(len(in_data), in_data))
        e_tag_0 = gr.tag_utils.python_to_tag(
            (0, pmt.intern("tx_sob"), pmt.PMT_T, pmt.PMT_NIL))
        e_tag_1 = gr.tag_utils.python_to_tag(
            (0, pmt.intern("tx_time"), tag_time, pmt.PMT_NIL))
        e_tag_2 = gr.tag_utils.python_to_tag(
            (len(in_data) - 1, pmt.intern("tx_eob"), pmt.PMT_T, pmt.PMT_NIL))

        self.tb.start()
        self.p2s.to_basic_block()._post(pmt.intern("pdus"), pmt.intern("MALFORMED PDU"))
        self.p2s.to_basic_block()._post(pmt.intern("pdus"), in_pdu)
        self.waitFor(lambda: len(self.vs.tags()) == 3,
                     timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        tags = self.vs.tags()
        self.assertEqual(len(tags), 3)
        self.assertEqual(tags[0].offset, e_tag_0.offset)
        self.assertTrue(pmt.equal(tags[0].key, e_tag_0.key))
        self.assertTrue(pmt.equal(tags[0].value, e_tag_0.value))
        self.assertEqual(tags[1].offset, e_tag_1.offset)
        self.assertTrue(pmt.equal(tags[1].key, e_tag_1.key))
        self.assertTrue(pmt.equal(tags[1].value, e_tag_1.value))
        self.assertEqual(tags[2].offset, e_tag_2.offset)
        self.assertTrue(pmt.equal(tags[2].key, e_tag_2.key))
        self.assertTrue(pmt.equal(tags[2].value, e_tag_2.value))
        self.assertTrue((in_data == np.real(self.vs.data())).all())

    def test_003_timed_long(self):
        in_data = np.arange(25000).tolist()
        tag_time = pmt.make_tuple(
            pmt.from_uint64(11), pmt.from_double(0.123456))
        in_dict = pmt.dict_add(
            pmt.make_dict(), pmt.intern("tx_time"), tag_time)
        in_pdu = pmt.cons(in_dict, pmt.init_c32vector(len(in_data), in_data))
        e_tag_0 = gr.tag_utils.python_to_tag(
            (0, pmt.intern("tx_sob"), pmt.PMT_T, pmt.PMT_NIL))
        e_tag_1 = gr.tag_utils.python_to_tag(
            (0, pmt.intern("tx_time"), tag_time, pmt.PMT_NIL))
        e_tag_2 = gr.tag_utils.python_to_tag(
            (len(in_data) - 1, pmt.intern("tx_eob"), pmt.PMT_T, pmt.PMT_NIL))

        self.tb.start()
        self.p2s.to_basic_block()._post(pmt.intern("pdus"), pmt.intern("MALFORMED PDU"))
        self.p2s.to_basic_block()._post(pmt.intern("pdus"), in_pdu)
        self.waitFor(lambda: len(self.vs.tags()) == 3,
                     timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        tags = self.vs.tags()
        self.assertEqual(len(tags), 3)
        self.assertEqual(tags[0].offset, e_tag_0.offset)
        self.assertTrue(pmt.equal(tags[0].key, e_tag_0.key))
        self.assertTrue(pmt.equal(tags[0].value, e_tag_0.value))
        self.assertEqual(tags[1].offset, e_tag_1.offset)
        self.assertTrue(pmt.equal(tags[1].key, e_tag_1.key))
        self.assertTrue(pmt.equal(tags[1].value, e_tag_1.value))
        self.assertEqual(tags[2].offset, e_tag_2.offset)
        self.assertTrue(pmt.equal(tags[2].key, e_tag_2.key))
        self.assertTrue(pmt.equal(tags[2].value, e_tag_2.value))
        self.assertTrue((in_data == np.real(self.vs.data())).all())


if __name__ == '__main__':
    gr_unittest.run(qa_pdu_to_bursts)



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy as np

from gnuradio import gr, gr_unittest, blocks, fec
from gnuradio.fec.extended_encoder import extended_encoder
from gnuradio.fec.polar.encoder import PolarEncoder
from gnuradio.fec.polar import channel_construction as cc

# import os
# print('PID:', os.getpid())
# raw_input('tell me smth')


class test_polar_encoder_systematic(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_setup(self):
        block_size = 16
        num_info_bits = 8
        frozen_bit_positions = cc.frozen_bit_positions(
            block_size, num_info_bits, 0.0)

        polar_encoder = fec.polar_encoder_systematic.make(
            block_size, num_info_bits, frozen_bit_positions)

        self.assertEqual(block_size, polar_encoder.get_output_size())
        self.assertEqual(num_info_bits, polar_encoder.get_input_size())
        self.assertFloatTuplesAlmostEqual(
            (float(num_info_bits) / block_size, ), (polar_encoder.rate(), ))
        self.assertFalse(polar_encoder.set_frame_size(10))

    def test_002_work_function_packed(self):
        block_size = 256
        num_info_bits = block_size // 2

        data, ref, polar_encoder = self.get_test_data(
            block_size, num_info_bits, 1)
        src = blocks.vector_source_b(data, False)
        enc_block = extended_encoder(polar_encoder, None, '11')
        snk = blocks.vector_sink_b(1)

        self.tb.connect(src, enc_block, snk)
        self.tb.run()

        res = np.array(snk.data()).astype(dtype=int)
        self.assertTupleEqual(tuple(res), tuple(ref))

    def test_004_big_input(self):
        num_blocks = 30
        block_size = 1024
        num_info_bits = block_size // 8

        data, ref, polar_encoder = self.get_test_data(
            block_size, num_info_bits, num_blocks)
        src = blocks.vector_source_b(data, False)
        enc_block = extended_encoder(polar_encoder, None, '11')
        snk = blocks.vector_sink_b(1)

        self.tb.connect(src, enc_block, snk)
        self.tb.run()

        res = np.array(snk.data()).astype(dtype=int)
        self.assertTupleEqual(tuple(res), tuple(ref))

    def get_test_data(self, block_size, num_info_bits, num_blocks):
        # helper function to set up test data and together with encoder object.
        num_frozen_bits = block_size - num_info_bits
        frozen_bit_positions = cc.frozen_bit_positions(
            block_size, num_info_bits, 0.0)
        frozen_bit_values = np.array([0] * num_frozen_bits,)
        python_encoder = PolarEncoder(
            block_size,
            num_info_bits,
            frozen_bit_positions,
            frozen_bit_values)

        data = np.array([], dtype=int)
        ref = np.array([], dtype=int)
        for i in range(num_blocks):
            d = np.random.randint(2, size=num_info_bits)
            data = np.append(data, d)
            ref = np.append(ref, python_encoder.encode_systematic(d))
        polar_encoder = fec.polar_encoder_systematic.make(
            block_size, num_info_bits, frozen_bit_positions)
        return data, ref, polar_encoder


if __name__ == '__main__':
    gr_unittest.run(test_polar_encoder_systematic)



"""
Copyright 2021 Ettus Research, A National Instruments Brand.

SPDX-License-Identifier: GPL-3.0-or-later
"""

import sys

MAX_NUM_CHANNELS = 16

MAIN_TMPL = """\
id: uhd_rfnoc_window
label: RFNoC Window Block
category: '[Core]/UHD/RFNoC/Blocks'

parameters:
- id: num_chans
  label: Channel count
  dtype: int
  options: [ ${", ".join([str(n) for n in range(1, max_num_chans+1)])} ]
  default: 1
  hide: ${'$'}{ 'part' }
- id: block_args
  label: Block Args
  dtype: string
  default: ""
  hide: ${'$'}{ 'part' if not block_args else 'none'}
- id: device_select
  label: Device Select
  dtype: int
  default: -1
  hide: ${'$'}{ 'part' if device_select == -1 else 'none'}
- id: instance_index
  label: Instance Select
  dtype: int
  default: -1
  hide: ${'$'}{ 'part' if instance_index == -1 else 'none'}
${coeffs_params}
inputs:
- domain: rfnoc
  dtype: 'sc16'
  multiplicity: ${'$'}{num_chans}

outputs:
- domain: rfnoc
  dtype: 'sc16'
  multiplicity: ${'$'}{num_chans}

templates:
  imports: |-
    from gnuradio import uhd
  make: |-
    uhd.rfnoc_window(
        self.rfnoc_graph,
        uhd.device_addr(${'$'}{block_args}),
        ${'$'}{device_select},
        ${'$'}{instance_index})
${init_params}  callbacks:
${callback_params}

documentation: |-
  RFNoC Window Block:
  Applies a user defined window to the input sample stream which is
  effectively multiplying groups of samples with the provided coefficients.

  Channel count:
  Number of channels / streams to use with the RFNoC Window block. Note,
  this is defined by the RFNoC Window Block's FPGA build parameters and
  GNU Radio Companion is not aware of this value. An error will occur at
  runtime when connecting blocks if the number of channels is too large.

  Coefficient Type:
  Choice between floating point or integer coefficients.
  Floating point coefficients must be within range [-1.0, 1.0].
  Integer coefficients must be within range [-32768, 32767].

  Coefficients:
  Array of coefficients. Number of elements in the array implicitly sets
  the window length, and must be less than or equal to the maximum number
  of coefficients supported by the block.

file_format: 1
"""

COEFFS_PARAM = """- id: coeffs_type${n}
  label: 'Ch${n}: Coeffs Type'
  dtype: enum
  options: ['float', 'integer']
  default: 'float'
  hide: ${'$'}{ 'part' if num_chans > ${n} else 'all' }
- id: coeffs_float${n}
  label: 'Ch${n}: Coefficients'
  dtype: float_vector
  default: [1.0]
  hide: ${'$'}{ 'none' if coeffs_type${n} == 'float' and num_chans > ${n} else 'all' }
- id: coeffs_int${n}
  label: 'Ch${n}: Coefficients'
  dtype: int_vector
  default: [32767]
  hide: ${'$'}{ 'none' if coeffs_type${n} == 'integer' and num_chans > ${n} else 'all' }
"""

INIT_PARAM = """    ${'%'} if coeffs_type${n} == "float" and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_coefficients(${'$'}{coeffs_float${n}}, ${n})
    ${'%'} endif
    ${'%'} if coeffs_type${n} == "integer" and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_coefficients(${'$'}{coeffs_int${n}}, ${n})
    ${'%'} endif
"""

CALLBACKS_PARAM = """  - |
    ${'%'} if coeffs_type${n} == "float" and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_coefficients(${'$'}{coeffs_float${n}}, ${n})
    ${'%'} endif
  - |
    ${'%'} if coeffs_type${n} == "integer" and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_coefficients(${'$'}{coeffs_int${n}}, ${n})
    ${'%'} endif
"""


def parse_tmpl(_tmpl, **kwargs):
    """ Render _tmpl using the kwargs """
    from mako.template import Template

    block_template = Template(_tmpl)
    return str(block_template.render(**kwargs))


if __name__ == '__main__':
    file = sys.argv[1]
    coeffs_params = ''.join([
        parse_tmpl(COEFFS_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    init_params = ''.join([
        parse_tmpl(INIT_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    callback_params = ''.join([
        parse_tmpl(CALLBACKS_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    open(file, 'w').write(
        parse_tmpl(
            MAIN_TMPL,
            max_num_chans=MAX_NUM_CHANNELS,
            coeffs_params=coeffs_params,
            init_params=init_params,
            callback_params=callback_params
        )
    )



#!/usr/bin/env python
#
# Copyright 2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import cmath

from gnuradio import gr, gr_unittest, digital, blocks
from gnuradio.digital import psk


class test_costas_loop_cc(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test01(self):
        # test basic functionality by setting all gains to 0
        natfreq = 0.0
        order = 2
        self.test = digital.costas_loop_cc(natfreq, order)

        data = 100 * [complex(1, 0), ]
        self.src = blocks.vector_source_c(data, False)
        self.snk = blocks.vector_sink_c()

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.run()

        expected_result = data
        dst_data = self.snk.data()
        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 5)

    def test02(self):
        # Make sure it doesn't diverge given perfect data
        natfreq = 0.25
        order = 2
        self.test = digital.costas_loop_cc(natfreq, order)

        data = [complex(2 * random.randint(0, 1) - 1, 0) for i in range(100)]
        self.src = blocks.vector_source_c(data, False)
        self.snk = blocks.vector_sink_c()

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.run()

        expected_result = data
        dst_data = self.snk.data()

        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 5)

    def test03(self):
        # BPSK Convergence test with static rotation
        natfreq = 0.25
        order = 2
        self.test = digital.costas_loop_cc(natfreq, order)

        rot = cmath.exp(0.2j)  # some small rotation
        data = [complex(2 * random.randint(0, 1) - 1, 0) for i in range(100)]

        N = 40  # settling time
        expected_result = data[N:]
        data = [rot * d for d in data]

        self.src = blocks.vector_source_c(data, False)
        self.snk = blocks.vector_sink_c()

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.run()

        dst_data = self.snk.data()[N:]

        # generously compare results; the loop will converge near to, but
        # not exactly on, the target data
        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 2)

    def test04(self):
        # QPSK Convergence test with static rotation
        natfreq = 0.25
        order = 4
        self.test = digital.costas_loop_cc(natfreq, order)

        rot = cmath.exp(0.2j)  # some small rotation
        data = [
            complex(
                2 *
                random.randint(
                    0,
                    1) -
                1,
                2 *
                random.randint(
                    0,
                    1) -
                1) for i in range(100)]

        N = 40  # settling time
        expected_result = data[N:]
        data = [rot * d for d in data]

        self.src = blocks.vector_source_c(data, False)
        self.snk = blocks.vector_sink_c()

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.run()

        dst_data = self.snk.data()[N:]

        # generously compare results; the loop will converge near to, but
        # not exactly on, the target data
        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 2)

    def test05(self):
        # 8PSK Convergence test with static rotation
        natfreq = 0.25
        order = 8
        self.test = digital.costas_loop_cc(natfreq, order)

        rot = cmath.exp(-cmath.pi / 8.0j)  # rotate to match Costas rotation
        const = psk.psk_constellation(order)
        data = [random.randint(0, 7) for i in range(100)]
        data = [2 * rot * const.points()[d] for d in data]

        N = 40  # settling time
        expected_result = data[N:]

        rot = cmath.exp(0.1j)  # some small rotation
        data = [rot * d for d in data]

        self.src = blocks.vector_source_c(data, False)
        self.snk = blocks.vector_sink_c()

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.run()

        dst_data = self.snk.data()[N:]

        # generously compare results; the loop will converge near to, but
        # not exactly on, the target data
        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 2)


if __name__ == '__main__':
    gr_unittest.run(test_costas_loop_cc)



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy as np

from gnuradio import gr, gr_unittest, blocks, fec

from gnuradio.fec.extended_decoder import extended_decoder
from gnuradio.fec.polar.encoder import PolarEncoder
from gnuradio.fec.polar import channel_construction as cc

# import os
# print('PID:', os.getpid())
# raw_input('tell me smth')


class test_polar_decoder_sc_systematic(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_setup(self):
        block_size = 16
        num_info_bits = 8
        frozen_bit_positions = np.arange(block_size - num_info_bits)

        polar_decoder = fec.polar_decoder_sc_systematic.make(
            block_size, num_info_bits, frozen_bit_positions)

        self.assertEqual(num_info_bits, polar_decoder.get_output_size())
        self.assertEqual(block_size, polar_decoder.get_input_size())
        self.assertFloatTuplesAlmostEqual(
            (float(num_info_bits) / block_size, ), (polar_decoder.rate(), ))
        self.assertFalse(polar_decoder.set_frame_size(10))

    def test_002_one_vector(self):
        block_power = 4
        block_size = 2 ** block_power
        num_info_bits = block_size // 2
        frozen_bit_positions = cc.frozen_bit_positions(
            block_size, num_info_bits, 0.0)

        bits, gr_data = self.generate_test_data(
            block_size, num_info_bits, frozen_bit_positions, 1, False)

        polar_decoder = fec.polar_decoder_sc_systematic.make(
            block_size, num_info_bits, frozen_bit_positions)
        src = blocks.vector_source_f(gr_data, False)
        dec_block = extended_decoder(polar_decoder, None)
        snk = blocks.vector_sink_b(1)

        self.tb.connect(src, dec_block)
        self.tb.connect(dec_block, snk)
        self.tb.run()

        res = np.array(snk.data()).astype(dtype=int)
        self.assertTupleEqual(tuple(res), tuple(bits))

    def test_003_stream(self):
        nframes = 3
        block_power = 8
        block_size = 2 ** block_power
        num_info_bits = block_size // 2
        frozen_bit_positions = cc.frozen_bit_positions(
            block_size, num_info_bits, 0.0)

        bits, gr_data = self.generate_test_data(
            block_size, num_info_bits, frozen_bit_positions, nframes, False)

        polar_decoder = fec.polar_decoder_sc_systematic.make(
            block_size, num_info_bits, frozen_bit_positions)
        src = blocks.vector_source_f(gr_data, False)
        dec_block = extended_decoder(polar_decoder, None)
        snk = blocks.vector_sink_b(1)

        self.tb.connect(src, dec_block)
        self.tb.connect(dec_block, snk)
        self.tb.run()

        res = np.array(snk.data()).astype(dtype=int)
        self.assertTupleEqual(tuple(res), tuple(bits))

    def generate_test_data(
            self,
            block_size,
            num_info_bits,
            frozen_bit_positions,
            nframes,
            onlyones):
        frozen_bit_values = np.zeros(block_size - num_info_bits, dtype=int)
        encoder = PolarEncoder(
            block_size,
            num_info_bits,
            frozen_bit_positions,
            frozen_bit_values)
        bits = np.array([], dtype=int)
        data = np.array([], dtype=int)
        for n in range(nframes):
            if onlyones:
                b = np.ones(num_info_bits, dtype=int)
            else:
                b = np.random.randint(2, size=num_info_bits)
            d = encoder.encode_systematic(b)
            bits = np.append(bits, b)
            data = np.append(data, d)
        gr_data = 2.0 * data - 1.0
        return bits, gr_data


if __name__ == '__main__':
    gr_unittest.run(test_polar_decoder_sc_systematic)



#!/usr/bin/env python
#
# Copyright 2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt
import numpy


def make_tag(key, value, offset, srcid=None):
    tag = gr.tag_t()
    tag.key = pmt.string_to_symbol(key)
    tag.value = pmt.to_pmt(value)
    tag.offset = offset
    if srcid is not None:
        tag.srcid = pmt.to_pmt(srcid)
    return tag


class test_skiphead(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.src_data = [int(x) for x in range(65536)]

    def tearDown(self):
        self.tb = None

    def test_skip_0(self):
        skip_cnt = 0
        expected_result = self.src_data[skip_cnt:]
        src1 = blocks.vector_source_i(self.src_data)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)

    def test_skip_1(self):
        skip_cnt = 1
        expected_result = self.src_data[skip_cnt:]
        src1 = blocks.vector_source_i(self.src_data)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)

    def test_skip_1023(self):
        skip_cnt = 1023
        expected_result = self.src_data[skip_cnt:]
        src1 = blocks.vector_source_i(self.src_data)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)

    def test_skip_6339(self):
        skip_cnt = 6339
        expected_result = self.src_data[skip_cnt:]
        src1 = blocks.vector_source_i(self.src_data)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)

    def test_skip_12678(self):
        skip_cnt = 12678
        expected_result = self.src_data[skip_cnt:]
        src1 = blocks.vector_source_i(self.src_data)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)

    def test_skip_all(self):
        skip_cnt = len(self.src_data)
        expected_result = self.src_data[skip_cnt:]
        src1 = blocks.vector_source_i(self.src_data)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        self.assertEqual(expected_result, dst_data)

    def test_skip_tags(self):
        skip_cnt = 25
        expected_result = self.src_data[skip_cnt:]

        src_tags = tuple([make_tag('foo', 'bar', 1, 'src'),
                          make_tag('baz', 'qux', 50, 'src')])
        src1 = blocks.vector_source_i(self.src_data, tags=src_tags)
        op = blocks.skiphead(gr.sizeof_int, skip_cnt)
        dst1 = blocks.vector_sink_i()
        self.tb.connect(src1, op, dst1)
        self.tb.run()
        dst_data = dst1.data()
        dst_tags = dst1.tags()
        self.assertEqual(expected_result, dst_data)
        self.assertEqual(dst_tags[0].offset, 25, "Tag offset is incorrect")
        self.assertEqual(len(dst_tags), 1, "Wrong number of tags received")
        self.assertEqual(pmt.to_python(
            dst_tags[0].key), "baz", "Tag key is incorrect")
        self.assertEqual(pmt.to_python(
            dst_tags[0].value), "qux", "Tag value is incorrect")


if __name__ == '__main__':
    gr_unittest.run(test_skiphead)
